# -*- coding: utf-8 -*-
"""
Translates rinex3 to rinex2. relies on gfzrnx
"""
import argparse
import os
import subprocess
import sys

import gnssrefl.gps as g

def main():
    """
    Converts a RINEX 3 file into a RINEX 2.11 file. Uses gfzrnx.

    Parameters
    ----------
    rinex3 : str
        filename for RINEX 3 file
    rinex2 : str, optional
        filename for RINEX 2.11 file
    dec : integer, optional
        decimation value (seconds)
    gpsonly : bool, optional
        whether to remove everything except GPS. Default is False

    """

    parser = argparse.ArgumentParser()
    parser.add_argument("rinex3", help="rinex3 filename", type=str)
    parser.add_argument("-rinex2", help="rinex2 filename", type=str,default=None)
    parser.add_argument("-dec", help="decimation value (seconds)", type=int,default=None)
    parser.add_argument("-gpsonly", help="remove everything except GPS", type=str,default=None)

    args = parser.parse_args()
    rinex3 = args.rinex3

    gexe = g.gfz_version()

    # should use utils but don't have the time
    gpsonly = False
    if (args.gpsonly == 'True') or (args.gpsonly == 'T'):
        gpsonly = True

    if args.rinex2 is None:
        station = rinex3[0:4].lower()
        cyyyy = rinex3[12:16]
        cdoy = rinex3[16:19]
        rinex2 = station + cdoy + '0.' + cyyyy[2:4] + 'o'
        print('Output filename ', rinex2)
    else:
        rinex2 = args.rinex2

    if args.dec is None:
        dec = 1
    else:
        dec=args.dec

    if not os.path.exists(gexe):
        print('Required gfzrnx executable does not exist. Exiting.')
        sys.exit()


    if os.path.isfile(rinex3):
        print('Your RINEX input file does exist:', rinex3)
        if (rinex3[-2::] == 'gz'):
            print('Must gunzip version 3 rinex')
            subprocess.call(['gunzip', rinex3])
            rinex3 = rinex3[0:-3]
        g.new_rinex3_rinex2(rinex3,rinex2,dec,gpsonly)
    else:
        print('ERROR: your input file does not exist:', rinex3)
        sys.exit()

    if os.path.isfile(rinex2):
        print('SUCCESS: new file created', rinex2)

if __name__ == "__main__":
    main()



import datetime as datetime
import math
import matplotlib.pyplot as plt
import numpy as np
import os
import subprocess
import sys
from astropy.time import Time

import gnssrefl.gps as g

def trans_time(tvd, ymd, ymdhm, convert_mjd, ydoy ,xcol,ycol,utc_offset):
    """
    translates time for quickplt

    Parameters
    ----------
    tvd : numpy array 
        contents of whatever file was read by loadtxt in quickplt
    ymd : bool
        first three columns are year,month,day, hour,
        minute,second
    ymdhm : bool
        first five columns are year,month,day, hour, minut,
    convert_mjd : bool
        convert from MJD (column 1 designation)
        time is datetime obj
    ydoy : bool
        first two columns are year and day of year
        time is datetime obj
    xcol : int
        column number for x-axis in python speak
    ycol : int
        column number for y-axis in python speak
    utc_offset : int
        offst in hours from UTC/GPS time.  None means do not use

    Returns
    -------

    tval : numpy array
         time, via floats or datetime, depending on what was requested
    yval : numpy array
         floats  - whatever is being plotted on the yaxis
    """

    tval = []
    yval = []

    nr,nc = tvd.shape
    #print('rows and columns ', nr,nc)
    if (ycol+1 > nc):
        print('You asked to plot column', ycol+1, ' and that column does not exist in the file')
        sys.exit()
    if (xcol+1 > nc):
        print('You asked to plot column', xcol+1, ' and that column does not exist in the file')
        sys.exit()

    if ymd :
        yval = tvd[:,ycol] # ??
        for i in range(0,len(tvd)):
            bigT = datetime.datetime(year=int(tvd[i,0]), month=int(tvd[i,1]), day=int(tvd[i,2]) )
            tval.append(bigT)
    elif ymdhm:
        yval = tvd[:,ycol] # ??
        for i in range(0,len(tvd)):
            bigT = datetime.datetime(year=int(tvd[i,0]), month=int(tvd[i,1]), 
                                     day=int(tvd[i,2]), hour=int(tvd[i,3]), minute=int(tvd[i,4]), second=0)
            tval.append(bigT)
    else:
        if convert_mjd:
            mm = tvd[:,xcol]
            if utc_offset is not None :
                if utc_offset != 0:
                    print('Apply local time offset')
                    mm = mm + utc_offset*3600/86400
            t1 = Time(mm,format='mjd')
            t1_utc = t1.utc # change to UTC
            #if utc_offset is not None:

            # probably can be done in one step!
            tval =  t1_utc.datetime # change to datetime
            yval = tvd[:,ycol] # save the y values
        elif ydoy:
            tval = tvd[:,0]  + tvd[:,1]/365.25
            yval = tvd[:,ycol]
            ii = np.argsort( tval)
            tval = tval[ii]; yval=yval[ii]
        else:
            tval = tvd[:,xcol] ; yval = tvd[:,ycol]
            x1 = min(tval) ; x2 = max(tval)

    return tval, yval 

def set_xlimits_ydoy(xlimits):
    """
    translate command line xlimits into datetime for nicer plots

    Parameters
    ----------
    xlimits : list of floats
         year (fractional, i.e. 2015.5)

    Returns 
    -------
    t1 : datetime
         beginning date for x-axis
    t2 : datetime
         end date for x-axis
    """
    year1 = math.floor(xlimits[0])
    doy1= math.floor(365.25*(xlimits[0]-year1))
    if doy1 == 0:
        doy1 = 1
    yy, mm, dd = g.ydoy2ymd(year1,doy1)
    t1 = datetime.datetime(year=yy, month=mm, day=dd)

    year2 = math.floor(xlimits[1])
    doy2= math.floor(365.25*(xlimits[1]-year2))
    if doy2 == 0:
        doy2 = 1

    yy, mm, dd  = g.ydoy2ymd(year2,doy2)
    t2 = datetime.datetime(year=yy, month=mm, day=dd)

    return t1, t2


def save_plot(out):
    """
    Makes a png plot and saves it in REFL_CODE/Files

    Parameters
    ----------
    out : str
        name of output file (or None)

    """
    # make sure output directory exists.
    xdir = os.environ['REFL_CODE']  + '/Files/' 
    if not os.path.exists(xdir) :
        subprocess.call(['mkdir', xdir])

    if out is None:
        out = 'temp.png'
        print('Plotfile saved to: ', xdir + out)
        plt.savefig(xdir + out ,dpi=300)
    else:
        if out[-3:] == 'png':
            print('Plotfile saved to: ', xdir  + out)
            plt.savefig(xdir + out,dpi=300)
        else:
            print('Output filename must end in png. No file is being written')



# -*- coding: utf-8 -*-
"""
kristine larson
combine multiple years of teqc multipath metrics, 
write a file, and make a plot
"""
import argparse
import matplotlib.pyplot as plt
import numpy as np
import os
import subprocess
import sys
import time
import wget

import gnssrefl.gps as g
import gnssrefl.computemp1mp2 as veg
def in_winter(d):
    """(td testing autodoc api generation)

    pretty silly winter screen tool

    Parameters
    ----------
    d : int
        day of year

    Returns
    -------
    bool
        True if doy is in winter, False if not considered "winter".
    """
    inwinter = False
    if (d < 105) or (d > 274):
        inwinter = True
    return inwinter 

def newvegplot(vegout,station):
    """
    send the file name and try to make a plot segreating for 
    changes in teqc metric and receiver type
    """
    # get the numerical data
    tv = np.loadtxt(vegout,usecols=(0,1,2,3))
    # read the receiver type separately ... because it is a string
    r = np.genfromtxt(vegout,usecols=(4),dtype='str')
    rcvs = np.unique(r)
# number of receivers
    N = len(rcvs)
    colors = 'brybrybry'
    plt.figure()
    k=0
    for i in range(0,N):
        receiver = rcvs[i]
    # find the indices with the correct receiver
        ii = (receiver == r)
        outx = tv[ii,0] + tv[ii,1]/365.25
    # now segreate by those that are not zero ... because teqc changed
    # from using MP1 to MP12
        outcol2 = tv[ii,2]
        outcol3 = tv[ii,3]
        ii = (outcol2 > 0)
        jj = (outcol3 > 0)
    # since we have a legend we don't want to plot it when it is empty
        xout = np.append(outx[ii],outx[jj])
        yout = np.append(-outcol2[ii],-outcol3[jj])
        plt.plot(xout,yout,'.', label=receiver)
        #if (len(outx[ii]) > 0):
        #    plt.plot(outx[ii], -outcol2[ii],dd,label=receiver)
        #if (len(outx[jj]) > 0):
        #    plt.plot(outx[jj], -outcol3[jj],dd,label=receiver)
        k = k + 1
    plt.legend(loc="lower left")
    plt.ylabel('-L1 rms (m)')
    plt.grid()
    plt.title('L1 Multipath Statistics for ' + station.upper() )
    plt.show()

def vegoutfile(station):
    """
    make sure directories exist for prelim veg output file
    returns name of the otuput file
    """
    vegdir = os.environ['REFL_CODE'] + '/Files'
    if not os.path.isdir(vegdir):
        subprocess.call(['mkdir',vegdir])
    vegdir = vegdir + '/veg'
    if not os.path.isdir(vegdir):
        subprocess.call(['mkdir',vegdir])
    vegout =  vegdir + '/' + station + '_veg.txt'
    print('File will be written to: ', vegout)

    return vegout 


def main():
    """
    command line interface for download_rinex
    """

    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name", type=str)
    parser.add_argument("year1", help="beginning year", type=int)
    parser.add_argument("year2", help="end year", type=int)
    parser.add_argument("-rcvtype", default = None, help="Receiver type", type=str)
    parser.add_argument("-winter", default = None, help="Removes winter points", type=str)

    args = parser.parse_args()

#   make sure environment variables exist.  set to current directory if not
    g.check_environ_variables()

#   assign to normal variables
    station = args.station
    if len(station) != 4:
        print('illegal station - must be 4 char')
        sys.exit()

    vegout = vegoutfile(station)

    y1 = args.year1
    y2 = args.year2 + 1

    if args.rcvtype == None:
        # do not restrict as the default
        rcvtype = 'NONE'
    else:
        rcvtype = args.rcvtype

    # should add a header
    vegid = open(vegout,'w+')

    if args.winter == None:
        winterMask = False
    else:
        winterMask = True
    k = 0
    if winterMask:
        for y in range(y1,y2):
            for d in range(1,367):
                if not in_winter(d):
                    sfile = veg.sfilename(station, y, d)
                    if os.path.isfile(sfile):
                        mp12, mp1,requested_rcv,rcvinfile=veg.readoutmp(sfile,rcvtype)
                        if requested_rcv:
                            k+=1
                            yy,mm,dd= g.ydoy2ymd(y,d)
                            vegid.write("{0:4.0f} {1:3.0f} {2:s} {3:s}  {4:s} {5:2.0f} {6:2.0f} \n".format(y,d,mp12[0:6],mp1[0:6], rcvinfile,mm,dd))
    else:
        for y in range(y1,y2):
            for d in range(1,367):
                sfile = veg.sfilename(station, y, d)
                if os.path.isfile(sfile):
                    mp12, mp1,requested_rcv,rcvinfile=veg.readoutmp(sfile,rcvtype)
                    if requested_rcv:
                        k+=1
                        yy,mm,dd= g.ydoy2ymd(y,d)
                        vegid.write("{0:4.0f} {1:3.0f} {2:s} {3:s}  {4:s} {5:2.0f} {6:2.0f} \n".format(y,d,mp12[0:6],mp1[0:6], rcvinfile,mm,dd))
    vegid.close()
    print(k, ' daily observations')
    if k > 0:
        newvegplot(vegout,station)
if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import argparse
import numpy as np
import time
import pickle
import urllib
import os
import sys
import subprocess
import urllib

import gnssrefl.gps as g

def download_prn_gps():
    """
    downloads PRN to GPS name conversion file from JPL

    """
    url='https://sideshow.jpl.nasa.gov/pub/gipsy_products/gipsy_params/PRN_GPS.gz'
    file_name='PRN_GPS.gz'
    print('# Downloading the file from JPL')
  
    try:
        urllib.request.urlretrieve(url, file_name)
        subprocess.call(['gunzip',file_name])
    except:
        print('something in the download did not work.')

    return

def read_jpl_file(fname):
    """
    Parameters
    ----------
    fname : str
        filename with prn gps conversion info

    Returns
    -------
    tv : list
        start date (frac year), end date, GPS#, PRN #

    """
    tv = [] # in case there is some problem
    #fname = 'PRN_GPS'
    if os.path.isfile(fname):
        x=np.loadtxt(fname, usecols=(0,1,2,3), skiprows=1,dtype='str')
        N,nc = x.shape
    # date1, date2, gps, prn will go in the 4 columns
        tv = np.empty(shape=[0,4])
        for i in range(0,N):
            t1 = g.cdate2nums(x[i,0]) # fractional date
            t2 = g.cdate2nums(x[i,1])
            prn = int(x[i,3])
            gps = int(x[i,2])
        # only save if PRN < 33
            if (prn < 33) :
                newl = [t1, t2, gps, prn]
                tv = np.append(tv, [newl], axis=0)
    else:
        print('could not find the PRN to GPS conversion file ', fname)
        sys.exit()
    #pname = 'PRN_GPS.pickle'
    #write_jpl_pickle_file(pname,tv)
    return tv


def main():
    """
    Displays the PRN and SVN numbers for the GPS
    constellation on a given date.

    Parameters
    ----------
    date : str
        example 2012-01-01
    overwrite : bool, optional
        whether you want to download new file from JPL

    """

    parser = argparse.ArgumentParser()
    parser.add_argument("date", help="2012-12-05", type=str)
    parser.add_argument("-overwrite", help="set to T or True to download new PRN_GPS file", type=str)
    args = parser.parse_args()
    cdate = args.date
    seekTime =  g.cdate2nums(cdate)
    found = False
    fout = 'PRN_GPS'
    xdir = os.environ['REFL_CODE']
    if (args.overwrite == 'T') or (args.overwrite == 'True'):
        print('# Will remove existing file, if it exists locally, and will download a new one')
        subprocess.call(['rm','-f','PRN_GPS'])
        download_prn_gps()


    print('# Looking for the PRN_GPS file.')
    if os.path.isfile('PRN_GPS'):
         print('# Found locally: ', fout)
         found = True
    if not found:
         print('# Look in the Files directory')
         fout = xdir + '/Files/PRN_GPS'
         if os.path.isfile(fout):
             print('Found: ', fout)
             found = True
    if not found:
        print('# Try to download a file from JPL')
        download_prn_gps()
        if os.path.isfile('PRN_GPS'):
            found = True
            fout = 'PRN_GPS' # though should probably move it
            print('# Downloaded and stored locally: ',fout)

    if not found:
        print('No input file, no output')
        sys.exit()


    x = read_jpl_file(fout)


    #N,nc = x.shape
    # check all 32 PRN numbers

    xx = 0
    for rprn in range(1,33):
        xxx = x[ (rprn == x[:,3] ) & (seekTime >= x[:,0]) & (seekTime <= x[:,1] ), 2 ] 
        gps_name = xxx.astype(int)
        if gps_name > 0:
            xx = xx + 1
            print("%2.0f  %2.0f " % (rprn, gps_name[0]) )
    print('\n There were ', xx, ' valid GPS satellites on ', cdate)


if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
"""
quickLook command line function 
"""
import argparse
import sys

# internal codes
import gnssrefl.gps as g
# old version
#import gnssrefl.quickLook_function as quick
import gnssrefl.quickLook_function2 as quick2

from gnssrefl.utils import validate_input_datatypes, str2bool


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name", type=str)
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("doy", help="day of year", type=int)
    parser.add_argument("-snr", default=None, type=int, help="snr ending - default is 66")
    parser.add_argument("-fr", default=None, type=int, help="e.g. -fr 1 for GPS L1  or -fr 101 for Glonass L1")
    parser.add_argument("-ampl", default=None, type=float, help="minimum spectral amplitude allowed")
    parser.add_argument("-e1",  default=None, type=float, help="lower limit elevation angle (default is 5 deg) ")
    parser.add_argument("-e2",  default=None, type=float, help="upper limit elevation angle (default is 25 deg)")
    parser.add_argument("-h1",  default=None, type=float, help="lower limit reflector height (default is 0.5m) ")
    parser.add_argument("-h2",  default=None, type=float, help="upper limit reflector height (default is 8 m)")
    parser.add_argument("-azim1",  default=None, type=float, help="lower limit azimuth (default is 0 deg)")
    parser.add_argument("-azim2",  default=None, type=float, help="upper limit azimuth (default is 360 deg)")
    parser.add_argument("-sat", default=None, type=int, help="satellite")
    parser.add_argument("-screenstats", default=None, type=str, help="if True, Success and Failure info printed to the screen")
    parser.add_argument("-peak2noise",  default=None, type=float, help="Quality Control ratio (default is 3)")
    parser.add_argument("-ediff",  default=None, type=float, help="ediff Quality Control parameter (default 2 deg)")
    parser.add_argument("-delTmax",  default=None, type=float, help="maximum arc length, in minutes, (default is 75 )")
    parser.add_argument("-plt", default=None, type=str, help="Set to false to turn off plots to the screen.")
    parser.add_argument("-hires_figs", default=None, type=str, help="Set to true to make eps instead of png.")

    g.print_version_to_screen()


    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['screenstats','plt','hires_figs']
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def quicklook(station: str, year: int, doy: int,
              snr: int = 66, fr: int = 1, ampl: float = 7., e1: float = 5, e2: float = 25, h1: float = 0.5, 
              h2: float = 8., sat: int = None, peak2noise: float = 3., screenstats: bool = False, fortran: bool = None, 
              plt: bool = True, azim1: float = 0., azim2: float = 360., ediff: float = 2.0, delTmax : float=75.0, hires_figs : bool=False ):
    """

    quickLook assessment of GNSS-IR results using SNR data. It creates two plots: one with periodograms for
    four different quadrants (northwest, northeast, southeast, southwest) and the other with the RH
    results shown as a function of azimuth. This plot also summarizes why the RH retrievals were accepted
    or rejected in terms of the quality control parameters.  

    Examples
    --------
    quickLook p041 2023 1 
        analyzes station p041 on day of year 1 in the year 2023 with defaults (L1, e1=5, e2=25)

    quickLook p041 2023 1 -h1 1 -h2 10
        analyzes station p041 on day of year 1 in the year 2023.  
        The periodogram would be restricted to RH of 1-10 meters.  

    If your site name is in the GNSS-IR database (which is generated from the Nevada Reno geodesy group), 
    a standard refraction correction is applied. If not, it does not.  This is most relevant for very very
    tall sites, i.e. > 200 meters. Refraction models are always applied in the gnssir module.

    If users would like the refraction correction to be applied here for stations that are not in the standard database,
    they need to submit a pull request making that possible. One option is to read from an existing gnssir_input 
    json file. That is what is done in nmea2snr and invsnr_input.

    Parameters
    ----------
    station : str
        4 character ID of the station
    year : int
        Year
    doy : int
        Day of year
    snr : int, optional
        SNR format. This tells the code which SNR file to use.  66 is the default.
        Other options: 50, 88, and 99.

    f : int, optional. 
        GNSS frequency. Default is GPS L1
        value options:

            1,2,20,5 : GPS L1,L2,L2C,L5 (1 is default)

            101,102 : GLONASS L1 and L2

            201,205,206,207,208 : GALILEO E1 E5a E6,E5b,E5

            302,306,307 : BEIDOU B1, B3, B2

    reqAmp : int or array_like, optional
        Lomb-Scargle Periodogram (LSP) amplitude significance criterion in volts/volts.
        Default is 7 

    e1 : int, optional
        elevation angle lower limit in degrees for the LSP.
        default is 5.

    e2: int, optional
        elevation angle upper limit in degrees for the LSP.
        default is 25.

    h1 : float, optional
        The allowed LSP reflector height lower limit in meters.
        default is 0.5.

    h2 : float, optional
        The allowed LSP reflector height upper limit in meters.
        default is 6.

    sat : integer, optional
        specific satellite number, default is None.

    peak2noise : int, optional
        peak to noise ratio of the periodogram values (periodogram peak divided by the periodogram noise).
        For snow and ice, 3.5 or greater, tides can be tricky if the water is rough (and thus
        you might go below 3 a bit, say 2.7 
        default is 3.

    screenstats : boolean, optional
        Whether to print stats to the screen.
        default is False.

    plt : boolean, optional
        Whether to print plots to the screen.
        default is True. Regardless, png files are made

    azim1 : float, optional
        minimum azimuth angle (deg)
        default is 0.

    azim2 : float, optional
        maximum azimuth angle (deg)
        default is 360.

    ediff : float, optional
        elevation angle difference, quality control parameter 
        default is 2 degrees.

    delTmax: float, optional
        maximum allowed arc length, in minutes
        default is 75 minutes.

    hires_figs : bool, optional
        eps instead of png files
    """

    vers = 'gnssrefl version ' + str(g.version('gnssrefl'))
    #print('You are running ', vers)


#   make sure environment variables exist.  set to current directory if not
    g.check_environ_variables()

    # checks for output
    g.checkFiles(station, '')

    exitS = g.check_inputs(station, year, doy, snr)


    if exitS:
        sys.exit()

    # set some reasonable default values for LSP (Reflector Height calculation).
    # most of these can be overriden at the command line
    if fr is not type(list):
        fr = [fr]  # default is to do L1

    pele = [5, 30]  # polynomial fit limits
    if ampl is not type(list):
        ampl = [ampl]  # this is arbitrary  - but often true for L1 obs

    if e1 < 5:
        print('We have to change the polynomial limits because you went below 5 degrees.')
        print('This restriction is for quickLook only ')
        pele[0] = e1

    pltscreen = plt
    args = {'station': station.lower(), 'year': year, 'doy': doy, 'snr_type': snr, 'f': fr[0], 'reqAmp': ampl, 'e1': e1,
            'e2': e2, 'minH': h1, 'maxH': h2, 'PkNoise': peak2noise, 'satsel': sat, 'fortran': fortran, 'pele': pele,
            'pltscreen': pltscreen, 'screenstats': screenstats, 'azim1': azim1, 'azim2': azim2, 'ediff': ediff, 
            'delTmax': delTmax, 'hires_figs': hires_figs}

    deltaRH = h2-h1
    if (deltaRH <= 0):
        print('Your reflector height region is invalid (i.e. h1 is greater than h2). h1:', h1, ' h2: ',h2)
        print('Exiting')
        sys.exit()

    if (deltaRH < 5.5):
        print('The reflector height region must be at least 5.5 meters long. These values are used for computing ')
        print('a valid periodogram and for quality control.  Change h1 or h2 or both. Exiting')
        sys.exit()

    # returns two variables: data, datakey = quick.quicklook_function(**args)
    return quick2.quickLook_function(**args)


def main():
    args = parse_arguments()
    data, datakey = quicklook(**args)


if __name__ == "__main__":
    main()



import sys
import os
import numpy as np
import math
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
import subprocess
import warnings

import scipy.interpolate
import scipy.signal

import gnssrefl.gps as g
import gnssrefl.refraction as refr
import gnssrefl.rinex2snr as rinex
import gnssrefl.gnssir_v2 as gnssir_v2


def quickLook_function(station, year, doy, snr_type,f,e1,e2,minH,maxH,reqAmp,pele,satsel,PkNoise,fortran,
        pltscreen,azim1,azim2,ediff, delTmax,hires_figs,**kwargs):
    """
    This is the main function to compute spectral characteristics of a SNR file.
    It takes in all user inputs and calculates reflector heights. It makes two png files to summarize
    the data.


    This is a new version that tries to picks all rising and setting arcs, not just those constrained to 90 degree
    quadrants.

    Parameters
    ----------
    station : str
        station name (4 char) 
    year : int
        full year
    doy : int
        day of year  
    snr_type : int
        snr file extension (i.e. 99, 66 etc)
    f : int
        frequency (1, 2, 5), etc
    e1 : float
        minimum elevation angle in degrees
    e2 : float
        maximum elevation angle in degrees
    minH : float
        minimum allowed reflector height in meters
    maxH : float
        maximum allowed reflector height in meters
    reqAmp : float
        is LSP amplitude significance criterion
    pele : list of floats
        is the elevation angle limits for the polynomial removal.  units: degrees
    satsel : int
        satellite number
    PkNoise : float
        peak to noise ratio for QC
    fortran : bool
         whether external fortran translator is being explicitly called. 
    pltscreen : bool
        whether you want plots sent to the terminal
    azim1 : float
         minimum azimuth in degrees
    azim2 : float
         maximum azimuth in degrees
    ediff : float
         QC parameter - restricts length of arcs (degrees)
    delTmax : float
         maximum arc length in minutes
    hires_figs: bool
         whether to use eps instead of png

    """
    if f not in [1, 2, 20, 5, 101, 102, 201, 205, 206, 207, 208, 302,306]:
        print('As far as I know, this is an illegal frequency for this software. Exiting. ', f)
        sys.exit()
    # it really should only mean whether you want to run plt.show()
    if ((maxH - minH) < 5.5):
        print('Requested reflector heights (', minH, ',', maxH, ') are too close together. ')
        print('They must be at least 5.5 meters apart - and preferably further than that. Exiting')
        return

    testing = kwargs.get('test',False)
    if testing: 
        print('working on new strategy')

    screenstats = kwargs.get('screenstats',False)
    if screenstats:
        print('Some screen statistics will print to the screen')

    if (minH > maxH):
        print('Minimum RH',minH, ' cannot be greater than maximum RH.', maxH, ' Exiting.')
        return [],[]

    # dictionary for output
    data = {'NW':{},'SW':{},'NE':{},'SE':{},'fNW':{},'fSW':{},'fNE':{},'fSE': {} }

    # dictionary for key
    datakey = {'NW':{},'SW':{},'NE':{},'SE':{},'fNW':{},'fSW':{},'fNE':{},'fSE': {} }

    # make sure environment variables exist
    g.check_environ_variables()

    # make sure logs directory exists
    xdir = os.environ['REFL_CODE']
    logdir = xdir + '/logs'

    if not os.path.isdir(logdir):
        subprocess.call(['mkdir', logdir])

    # if it finds the station coordinates, it will return irefr as 1
    quick_p,quick_T,irefr, quick_e = quick_refraction(station)

    webapp = False 
    # orbit directories
    ann = g.make_nav_dirs(year)
    # titles in 4 quadrants - for webApp
    titles = ['Northwest', 'Southwest','Northeast', 'Southeast']
    stitles = ['NW', 'SW','NE', 'SE']
    # define where the axes are located
    bx = [0,1,0,1]; by = [0,0,1,1]; bz = [1,3,2,4]

    #  fontsizes
    fs = 10
    FS = 12
    # various defaults - ones the user doesn't change in this quick Look code
    polyV = 4 # polynomial order for the direct signal
    desiredP = 0.01 # 1 cm precision for a "quick Look"

    #four_in_one = True # put the plots together
    minNumPts = 20 
    #noise region for LSP QC. these are meters
    NReg = [minH, maxH]
    # for quickLook, we use the four geographic quadrants - these are azimuth angles in degrees
    azval = [270, 360, 180, 270, 0, 90, 90, 180]
    # try adding 5 degrees at the quadrant edges, except for north
    naz = int(len(azval)/2) # number of azimuth pairs
    pltname = 'temp.png' # default plot
    requireAmp = reqAmp[0]

    obsfile, obsfileCmp, snre =  g.define_and_xz_snr(station,year,doy,snr_type)
    allGood, snrD, nrows, ncols = gnssir_v2.read_snr(obsfile)

    if allGood == 0:
        print('file does not exist'); sys.exit()
    else:
        # make output file for the quickLook RRH values, just so you can give them a quick look see
        # also used in the azimuth QC plot
        quicklog = logdir + '/rh_' + station + '.txt'
        rhout = open(quicklog,'w+')
        amax = 0
        minEdataset = min(snrD[:,1])
        print('minimum elevation angle (degrees) for this dataset: ', minEdataset)
        if minEdataset > (e1+0.5):
            print('It looks like the receiver had an elevation mask. Overriding e1 to this value.')
            e1 = minEdataset

        # apply elevation angle correction for simple refraction here
        if irefr == 1:
            print('found simple refraction parameters')
            lsp={}; lsp['refraction'] = True
            snrD[:,1] = gnssir_v2.apply_refraction_corr(lsp,snrD[:,1],quick_p,quick_T)
        # restrict to DC limits 
        ele = snrD[:,1]


        i= (ele >= pele[0]) & (ele < pele[1])
        ele = ele[i]
        snrD = snrD[i,:]
        sats = snrD[:,0]

        # very rare that no GPS data exist, so will not check it.  I will probably regret this
        #if (f < 6):
        #    b = sats[(sats<100)]; print('gps',len(b))
        if (f > 100) and (f < 200): # glonass
            b = sats[(sats>100) & (sats<200)]; # print('glonass',len(b))
            if len(b) == 0:
                print('NO Glonass DATA in the file') ; 
                return data, datakey
        elif (f > 200) & (f < 300): # galileo
            b = sats[(sats>200) & (sats<300)]; # print('galileo',len(b))
            if len(b) == 0:
                print('NO Galileo data in the file'); 
                return data, datakey
        elif (f > 300): # beidou
            b = sats[(sats>300) & (sats<400)]; # print('beidou',len(b))
            if len(b) == 0:
                print('NO Beidou data in the file'); 
                return data, datakey

        fig = plt.figure(figsize=(10,6))
        ax1=plt.subplot(2,2,bz[0]); ax2=plt.subplot(2,2,bz[1]) ; ax3=plt.subplot(2,2,bz[2]) ; ax4=plt.subplot(2,2,bz[3])
        # save the handles in a dictionary
        saxis = {} ; saxis['0'] = ax1 ; saxis['1'] = ax2; saxis['2'] = ax3 ; saxis['3'] = ax4

        allpoints = 0

        # keep track of the maximum amplitude for each track in each quadrant so you can have them be hte same
        axisSize =np.empty(shape=[0, 2])

        if (satsel == None):
            satlist = gnssir_v2.find_mgnss_satlist(f,year,doy)
        else:
            satlist = [satsel]

        azvalues = [0,360] # for the start

        for satNu in satlist:
            iii = (sats == satNu)
            thissat = snrD[iii,:]
            goahead = False

            if len(thissat) > 0:
                # if there are data for this satellite, find all the arcs
                arclist = gnssir_v2.new_rise_set(thissat[:,1],thissat[:,2],thissat[:,3],e1, e2,ediff,satNu,screenstats)
                nr,nc = arclist.shape
                if nr > 0:
                    goahead = True
                else:
                    goahead = False

                if goahead:
                    for arc in range(0,nr):
                        sind = int(arclist[arc,0]) ; eind = int(arclist[arc,1])
                        d2 = np.array(thissat[sind:eind, :], dtype=float)
                    # window the data - which also removes DC
                        x,y, Nvv, cf, meanTime,avgAzim,outFact1, Edot2, delT= gnssir_v2.window_new(d2, f,
                                satNu,ncols,pele, polyV,e1,e2,azvalues,screenstats)
                        Nv = Nvv # number of points
                        UTCtime = meanTime
                    # for this arc, which a value is it?
                        a = whichquad(avgAzim)
                        #print('Looking at arc ', arc, ' quad ', a, avgAzim)
                        tooclose = False
                        if (delT != 0) & (Nv > 20):
                            T = g.nicerTime(UTCtime)
                            maxF, maxAmp, eminObs, emaxObs,riseSet,px,pz= g.strip_compute(x,y,cf,maxH,desiredP,polyV,minH)
                            if (maxF ==0) & (maxAmp == 0):
                                tooclose == True; Noise = 1; iAzim = 0;
                            else:
                                nij =  pz[(px > NReg[0]) & (px < NReg[1])]

                            Noise = 1
                            if (len(nij) > 0):
                                Noise = np.mean(nij)

                            iAzim = int(avgAzim)

                            if abs(maxF - minH) < 0.10: #  peak too close to min value
                                tooclose = True

                            if abs(maxF - maxH) < 0.10: #  peak too close to max value
                                tooclose = True

                            if (not tooclose) & (delT < delTmax) & (maxAmp > requireAmp) & (maxAmp/Noise > PkNoise) & (iAzim >= azim1) & (iAzim <= azim2):
                                rhout.write('{0:3.0f} {1:6.3f} {2:3.0f} {3:4.1f} {4:3.1f} {5:6.2f} {6:2.0f} \n '.format(iAzim,maxF,satNu,
                                    maxAmp,maxAmp/Noise,UTCtime,1))
                                lw=1.5 ; colorful(a,px,pz,lw,True,saxis)
                                idc = stitles[a]
                                data[idc][satNu] = [px,pz]
                                datakey[idc][satNu] = [avgAzim, maxF, satNu,f,maxAmp,maxAmp/Noise, UTCtime]
                                if screenstats:
                                    print('SUCCESS for Azimu {0:5.1f} Satellite {1:2.0f} UTC {2:5.2f} RH {3:7.3f} PkNoise {4:6.2f}'.format( avgAzim,satNu,UTCtime,maxF,maxAmp/Noise))
                                # try to track the maximum amplitude in each quadrant this way
                                newl=[a,maxAmp]; axisSize =np.append(axisSize,[newl], axis=0)

                            else:
                                if (iAzim > azim1) & (iAzim < azim2):
                                    lw = 0.5 ; colorful(a,px,pz,lw,False,saxis) # add to the plot
                                    if screenstats:
                                        print('FAILED QC for Azimuth {0:5.1f} Satellite {1:2.0f} UTC {2:5.2f} RH {3:7.3f} '.format( avgAzim,satNu,UTCtime,maxF))
                                        logout = None
                                        g.write_QC_fails(delT,delTmax,eminObs,emaxObs,e1,e2,ediff,maxAmp, Noise,PkNoise,requireAmp,tooclose,logout)

                                    idc = 'f' + stitles[a]
                                    data[idc][satNu] = [px,pz]
                                    datakey[idc][satNu] = [avgAzim, maxF, satNu,f,maxAmp,maxAmp/Noise, UTCtime]
                                    rhout.write('{0:3.0f} {1:6.3f} {2:3.0f} {3:4.1f} {4:3.1f} {5:6.2f} {6:2.0f} \n '.format(iAzim,maxF,
                                        satNu,maxAmp,maxAmp/Noise,UTCtime,-1))

        rhout.close()

        # put on all the labels ... hopefully
        set_labels(saxis,axisSize,fs)

        # make sure directory exists for plots
        g.set_subdir(station)
        # where plots will go
        fdir = os.environ['REFL_CODE'] + '/Files/' + station 
        tt = 'GNSS-IR: ' + station.upper() + ' Freq:' + g.ftitle(f) + ' Year/DOY:' + str(year) + ',' + str(doy) + ' elev: ' + str(e1) + '-' +  str(e2)
        fig.suptitle(tt, fontsize=FS)
        # if you have no results, no point plotting them!
        if hires_figs:
            filename = fdir + '/quickLook_lsp.eps'
        else:
            filename = fdir + '/quickLook_lsp.png'
        print('Plot saved to ', filename)

        plt.savefig(filename)
        # now make second plot
        goodbad(quicklog,station,year,doy,minH,maxH,PkNoise,reqAmp,f,e1,e2,hires_figs)

        if pltscreen:
            plt.show()

    return data,datakey

def goodbad(fname,station,year,doy,h1,h2,PkNoise,reqAmp,freq,e1,e2,hires_figs):
    """
    makes a plot that shows "good" and "bad" refletor height retrievals as a 
    function of azimuth

    Parameters
    ----------
    fname : str
        filename
    station : str
        4 char station name
    year : int
        full year
    doy : int
        day of year
    h1 : float
        minimum reflector height (m)
    h2 : float
        max reflector height (m)
    PkNoise : float
        peak 2 noise QC
    reqAmp : float
        required LSP amplitude
    freq : int
        frequency
    e1 : float
        minimum elevation angle (deg)
    e2 : float
        maximum elevation angle (deg)
    hires_figs : bool
        whether to use eps instead of png

    Returns
    -------
    plot is written $REFL_CODE/Files/station/quickLook_summary.png

    """
    try:
        # added this to get rid of the warning when the file has no bad points in it
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            a = np.loadtxt(fname,comments='%')
    except:
        print('No results in the file')

    #print(a.ndim, len(a))
    if (a.ndim == 1) or (len(a) == 0):
        print('No results in the file')
        return

    ij = (a[:,6] == 1) # good retrievals
    ik = (a[:,6] == -1) # bad retrievals
    fs = 12
    plt.figure(figsize=(10,6))
    plt.subplot(3,1,1)
    plt.plot(a[ij,0], a[ij,1], 'o',color='blue',label='good')
    plt.plot(a[ik,0], a[ik,1], 'o',color='gray', label='bad')
    ydoy = ' ' + str(year) + '/' + str(doy) + ' '
    #print(freq, g.ftitle(freq))
    plt.title('quickLook Retrieval Metrics: ' + station + ' Freq:' + g.ftitle(freq) + ydoy + ' elev:' + str(e1) + '-' + str(e2) ,fontsize=fs)
    plt.legend(loc="upper right")
    plt.ylabel('Refl. Ht. (m)',fontsize=fs)
    plt.grid()
    plt.xlim((0, 360))
    plt.ylim((h1, h2))
    plt.xticks(fontsize=fs)
    plt.yticks(fontsize=fs)

    ax = plt.gca()
    ax.axes.xaxis.set_ticklabels([])


    plt.subplot(3,1,2)
    plt.plot([0, 360], [PkNoise, PkNoise], 'k--',label='QC value used')
    plt.plot(a[ij,0], a[ij,4], 'o',color='blue')
    plt.plot(a[ik,0], a[ik,4], 'o',color='gray')
    plt.legend(loc="upper right")
    plt.ylabel('peak2noise',fontsize=fs)
    plt.grid()
    plt.xlim((0, 360))
    plt.xticks(fontsize=fs)
    plt.yticks(fontsize=fs)
    ax = plt.gca()
    ax.axes.xaxis.set_ticklabels([])

    plt.subplot(3,1,3)
    plt.plot([0, 360], [reqAmp, reqAmp], 'k--',label='QC value used')
    plt.plot(a[ij,0], a[ij,3], 'o',color='blue')
    plt.plot(a[ik,0], a[ik,3], 'o',color='gray') 
    plt.legend(loc="upper right")
    plt.ylabel('Spectral Peak Ampl.',fontsize=fs)
    plt.xlabel('Azimuth (degrees)',fontsize=fs)
    plt.grid()
    plt.xticks(fontsize=fs)
    plt.yticks(fontsize=fs)
    plt.xlim((0, 360))

    # existence of the output directory is checked earlier
    fdir = os.environ['REFL_CODE'] + '/Files/' + station 
    if hires_figs:
        f = fdir + '/quickLook_summary.eps'
    else:
        f = fdir + '/quickLook_summary.png'

    print('Plot saved to ', f)
    plt.savefig(f)

def colorful(a,px,pz,lw,fullcolor,ax):
    """
    plots the quadrant periodograms

    Parameters
    ----------
    a : int
    px : numpy array of floats
        x axis of amplitude power spectrum
    pz : numpy array of floats
        y axis of amplitude power spectrum
    lw : float
        line width
    fullcolor :
        whether you want full color (if not, it is gray)
    ax : axis handle
    """
    # which quadrant
    axisV = ax[str(a)]

    if fullcolor:
        axisV.plot(px,pz,linewidth=lw)
    else:
        axisV.plot(px,pz,'gray',linewidth=lw)


def set_labels(ax,axisSize,fs):
    """
    try to set the appropriate labels depending on the quadrant

    Parameters
    ----------
    ax : dictionary
        plot handles
    axisSize : numpy array of floats
        lists the amplitudes of the various periodograms in a given quadrant
        pairs of quad (0-3), amplitude
    fs : int
        font size
    """
    setA = False # want to set the axes based on good retrieval amplitudes
    # start maxv with at least something
    maxv=2
    for a in range(0,4):
        axisV = ax[str(a)]

        axisV.grid()
        if (a == 3) or (a==1):
            axisV.set_xlabel('reflector height (m)',fontsize=fs)
        if (a == 1) or (a==0):
            axisV.set_ylabel('volts/volts',fontsize=fs)

        ii = (axisSize[:,0] == a)
        thisquad = axisSize[ii,1]
        if len(thisquad) > 0:
            # found at least one good periodogram 
            setA = True
            thismax = max(thisquad)
            if (thismax > maxv):
                maxv = thismax
    # now apply it
    maxv = maxv*1.1
    if setA:
        for a in range(0,4):
            axisV = ax[str(a)]
            axisV.set_ylim(0,maxv)  

def whichquad(iaz):
    """
    Parameters
    ----------
    iaz : float
        azimuth of the arc, degrees
    a : int
        quad number in funny system
        
    """
    if (iaz >= 270):
        a = 0
    elif (iaz >= 180) & (iaz < 270):
        a = 1
    elif (iaz >= 0) & (iaz < 90):
        a = 2
    elif (iaz >= 90) &(iaz < 180):
        a =3

    return a

def quick_refraction(station):
    """
    refraction correction used in quickLook. no time dependence.

    Parameters
    ----------
    station : string
        4 character station name

    Returns
    -------
    p : float
        pressure, hPa
    T : float
        temperature, Celsius
    irefr : int
        refraction model number I believe, which is also sent, so not needed
    e : float
        water vapor pressure, hPa

    """

    xdir = os.environ['REFL_CODE']
    refraction_model = 1
    lat,lon,ht=g.queryUNR_modern(station.lower())
    # default values
    p = 0; T = 0; irefr = 1; e=0 ; it = 1 #?
    dmjd = 0
    if (lat == 0) & (lon == 0):
        #print('no coordinates found')
        irefr = 0
        return p,T,irefr, e

    refr.readWrite_gpt2_1w(xdir, station, lat, lon)
    dlat = lat*math.pi/180; dlong = lon*math.pi/180;
    p,T,dT,Tm,e,ah,aw,la,undu = refr.gpt2_1w(station, dmjd,dlat,dlong,ht,it)

    return p,T,irefr, e




import argparse
import matplotlib.pyplot as matplt
import os
import sys

import gnssrefl.daily_avg as da
import gnssrefl.gps as g
import gnssrefl.gnssir_v2 as guts2

from gnssrefl.utils import str2bool

def parse_arguments():
    # must input start and end year
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name (4 ch only)", type=str)
    parser.add_argument("medfilter", help="Median filter for daily RH (m). Start with 0.25 ", type=float)
    parser.add_argument("ReqTracks", help="required number of tracks", type=int)
    parser.add_argument("-txtfile", default=None, type=str, help="Set your own output filename")
    parser.add_argument("-plt", default=None, type=str, help="plt to screen: True or False")
    parser.add_argument("-extension", default=None, type=str, help="extension for solution names")
    parser.add_argument("-year1", default=None, type=int, help="restrict to years starting with")
    parser.add_argument("-year2", default=None, type=int, help="restrict to years ending with")
    parser.add_argument("-fr", default=0, type=int, help="frequency, default is to use all")
    parser.add_argument("-csv", default=None, type=str, help="True if you want csv instead of plain text")
    parser.add_argument("-azim1", default=None, type=int, help="minimum azimuth (deg)")
    parser.add_argument("-azim2", default=None, type=int, help="maximum azimuth (deg)")
    parser.add_argument("-test", default=None, type=str, help="augmentation to plot")
    parser.add_argument("-subdir", default=None, type=str, help="non-default subdirectory for output ")
    parser.add_argument("-plot_limits", default=None, type=str, help="add median value and limits to plot, default is False ")
    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['plt', 'csv','test','plot_limits']
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def daily_avg(station: str , medfilter: float, ReqTracks: int, txtfile: str = None, plt: bool = True, 
        extension: str = '', year1: int = 2005, year2: int = 2030, fr: int = 0, csv: bool = False, 
        azim1: int = 0, azim2: int = 360, test: bool = False, subdir: str=None,plot_limits: bool=False):
    """
    The goal of this code is to consolidate individual RH results into a single file consisting of 
    daily averaged RH without outliers. These daily average values are nominally associated 
    with the time of 12 hours UTC.


    There are two required parameters - medfilter and ReqTracks. These are quality control parameters.
    They are applied in two steps. The code first calculates the median value each day - and keeps
    only the RH that are within medfilter (meters) of this median value. If there are at least "ReqTracks"
    number of RH left after that step, a daily average is computed for that day.


    As of version 3.1.3 users may store the required input parameters to daily_avg 
    in the json used by gnssir. The names of these parameters are:
    daily_avg_reqtracks and  daily_avg_medfilter.  For those making a new json, 
    the parameters will be set to None if you don't choose a value on the command line. 
    You can also hand edit or add it. This would be helpful in not having to rerun gnssir_input and 
    risk losing some of your other specialized selections.

    If you are unfamiliar with what a median filter does in this code, please see 
    https://gnssrefl.readthedocs.io/en/latest/pages/README_dailyavg.html

    The outputs are stored in $REFL_CODE/Files/station by default.  If you want to specify a new
    subdirectory, I believe that is an allowed option.  You can also specify specific years to analyze 
    and apply fairly simple azimuth constraints.
    
    In summary, three text files are created

    1. individual RH values with no QC applied
    
    2. individual RH values with QC applied

    3. daily average RH 

    Examples
    -------- 
    daily_avg p041 0.25 10 
        consolidates results for p041 with median filter of 0.25 meters and at least 10 solutions per day
    daily_avg p041 0.25 10 -plot_limits T
        the same as above but with plot_limits to help you see where the median filter is applied
    daily_avg p041 0.25 10  -year1 2015 -year2 2020
        consolidates results for p041 with median filter of 0.25 meters and at least 10 solutions per day
        and restricts it to years between 2015 and 2020
    daily_avg p041 0.25 10  -year1 2015 -year2 2020 -azim1 0 -azim2 180
        consolidates results for p041 with median filter of 0.25 meters and at least 10 solutions per day
        and restricts it to years between 2015 and 2020 and azimuths between 0 and 180 degrees
    daily_avg p041 0.25 10 -extension NV
        consolidates results which were created using the extension NV when you ran gnssir.


    Parameters
    ----------
    station : str
        4 ch station name, generally lowercase

    medfilter : float
        Median filter for daily reflector height (m). Start with 0.25 for surfaces where you expect no significant 
        subdaily change (snow/lakes).

    ReqTracks : int
        Required number of daily satellite tracks to save the daily average value.

    txtfile : str, optional
        Use this parameter to set your own output filename.
        default is to let the code choose.

    plt : bool, optional
        whether to print plots to screen or not.
        default is True.

    extension : str, optional
        extension for solution names.
        default is ''. (empty string)

    year1 : int, optional
        restrict to years starting with.
        default is 2005.

    year2 : int, optional
        restrict to years ending with.
        default is 2030.

    fr : int, optional
        GNSS frequency. If none input, all are used. Value options:

            1 : GPS L1

            2 : GPS L2

            20 : GPS L2C

            5 : GPS L5

            101 : GLONASS L1

            102 : GLONASS L2

            201 : GALILEO E1

            205 : GALILEO E5a

            206 : GALILEO E6

            207 : GALILEO E5b

            208 : GALILEO E5

            302 : BEIDOU B1

            306 : BEIDOU B3

            307 : BEIDOU B2

    csv : boolean, optional
        Whether you want csv instead of a plain text file.
        default is False.

    azim1 : int, optional
        minimum azimuth, degrees
        note: should be modified to allow negative azimuth

    azim2 : int, optional
        maximum azimuth, degrees

    test : bool, optional
        not sure what this does

    subdir: str, optional
        non-default subdirectory for Files output

    plot_limits: bool, optional
        adds the median value and median filter limits to the plot.
        default is False

    """
    if len(station) != 4:
        print('Station names must have four characters. Exiting.')
        sys.exit()

    plt2screen = plt # since variable was originally this name 
    # make sure environment variables are set
    g.check_environ_variables()
    if subdir is None:
        subdir = station
    g.set_subdir(subdir)

# where the summary files will be written to
    xdir = os.environ['REFL_CODE']
    txtdir = xdir + '/Files/'  + subdir

    # read in potential medfilter and reqtracks values
    # set noexit to True because I want to look for the json but 
    # don't want to require it
    lsp = guts2.read_json_file(station, extension,noexit=True)
    # if valeus not found, then set them to None
    if 'daily_avg_medfilter' not in lsp:
        lsp['daily_avg_medfilter'] = None
    if 'daily_avg_reqtracks' not in lsp:
        lsp['daily_avg_reqtracks'] = None

    # if these required values are zero, then the code 
    # looks to see if you set them in the json
    if ReqTracks == 0:
        if  lsp['daily_avg_reqtracks'] is not None:
            print('Using ReqTracks value from json')
            ReqTracks = lsp['daily_avg_reqtracks']
        else:
            print('Zero is an invalid value for ReqTracks. Change value on command line')
            print('or in gnssir_input created json. Exiting.')
            sys.exit()
    if medfilter == 0:
        if  lsp['daily_avg_medfilter'] is not None:
            print('Using medfilter value from json')
            medfilter = lsp['daily_avg_medfilter']
        else:
            print('Zero is an invalid value for medfilter parameter. Change value on command ')
            print('line or in gnssir_input created json. Exiting.')
            sys.exit()

    # set the name of the output format
    if csv:
        alldatafile = txtdir + '/' + station + '_allRH.csv' 
    else:
        alldatafile = txtdir + '/' + station + '_allRH.txt' 

    # read in the files
    tv, obstimes = da.readin_plot_daily(station, extension, year1, year2, fr, 
            alldatafile, csv, medfilter, ReqTracks,azim1,azim2,test,subdir,plot_limits)

    # default is to show the plots
    nr,nc = tv.shape
    if plt2screen & (nr > 0):
        matplt.show()

    # now write out the result file:

    if txtfile is None:
        if csv:
            outfile = txtdir + '/' + station + '_dailyRH.csv' 
        else:
        # use default  filename for the average
            outfile = txtdir + '/' + station + '_dailyRH.txt' 
    else:
        # use filename provided by the user
        outfile = txtdir + '/' + txtfile

    if (nr > 0):
        da.write_out_RH_file(obstimes, tv, outfile, csv,station,extension)


def main():
    args = parse_arguments()
    daily_avg(**args)


if __name__ == "__main__":
    main()



import matplotlib.pyplot as plt
import numpy as np
import os
import subprocess
import sys


import gnssrefl.gps as g
import gnssrefl.read_snr_files as read_snr
from gnssrefl.utils import FileManagement, FileTypes
import gnssrefl.daily_avg_cl as da
import gnssrefl.gnssir_v2 as gnssir

from functools import partial
from scipy import optimize
from scipy.interpolate import interp1d
from datetime import datetime
from pathlib import Path

from gnssrefl.utils import str2bool, read_files_in_dir

xdir = Path(os.environ["REFL_CODE"])

def normAmp(amp, basepercent):
    """
    emulated amp_normK code from PBO H2O
    inputs are the amplitudes  and a percentage
    used to define the bottom level. returns
    normalized amplitudes
    this is meant to be used by individual tracks (I think)
    in this case they are the top values, not the bottom ... ugh
    """
    Namp = amp
    N=len(amp)
    if (N > 0):
        numx = int(np.round(N*basepercent) )

        if (numx==0):
            numx = N

        sortA = -np.sort(-amp)
        topavg = np.mean(sortA[0:numx])

        # this comes from Clara's Recommendations, amp_normK.m
        Namp = amp/topavg

    return Namp

def daily_phase_plot(station, fr,datetime_dates, tv,xdir,subdir,hires_figs):
    """
    makes a plot of daily averaged phase for vwc code

    Parameters
    ----------
    station: str
        4 char station name
    fr : int
        frequency of signal
    datetime_dates : ...
        datetime values for phase points
    tv : list of results
        cannot remember the format
    xdir : str
        location of the results (environment variable REFL_CODE)
    subdir : str
        subdirectory in Files
    hires_figs: bool
        whether you want eps instead of png files

    """
    outdir = xdir + '/Files/' + subdir
    plt.figure(figsize=(10, 6))
    plt.plot(datetime_dates, tv[:, 2], 'b-')
    plt.ylabel('phase (degrees)')
    if fr == 1:
        plt.title(f"Daily L1 Phase Results: {station.upper()}")
    else:
        plt.title(f"Daily L2C Phase Results: {station.upper()}")
    plt.grid()
    plt.gcf().autofmt_xdate()

    # maybe this works.  Maybe not.
    if hires_figs:
        plot_path = f'{outdir}/{station}_daily_phase.eps'
    else:
        plot_path = f'{outdir}/{station}_daily_phase.png'
    print(f"Saving figure to {plot_path}")

    plt.savefig(plot_path)



def make_snow_filter(station, medfilter, ReqTracks, year1, year2):
    """
    Runs daily_avg code to make a snow mask file.  This is 
    so you have some idea of when the soil moisture products are 
    contaminated by snow. Make a file with these years and doys saved. The user can
    edit if they feel the suggestsions are poor (i.e. days in the summer might show up
    as "snow")

    If snow mask file exists, it does not overwrite it.

    Parameters
    ----------
    station : str
        4 ch station name
    medfilter : float
        how much you allow the individual tracks to deviate from the daily median (meters)
    ReqTracks : int
        number of tracks to compute trustworthy daily average
    year1 : int
        starting year
    year2 : int
        ending year

    Returns
    -------
    snowmask_exists : bool
        whether file was created
    snow_file : str
        name of the snow mask file

    Creates output file into a file $REFL_CODE/Files/{ssss}/snowmask_{ssss}.txt
    where ssss is the station name

    """
    snowmask_exists = False
    myxdir = os.environ['REFL_CODE']
    snowfile = myxdir + '/Files/' + station + '/snowmask_' + station + '.txt' 
    if os.path.exists(snowfile):
        print('Using existing snow mask file, ', snowfile)
        snowmask_exists = True
        return snowmask_exists,snowfile

    # l2c only
    txtfile = station + '_allRH.txt'; 
    pltit = False; extension = ''; fr = 20; csv = False

    # writes out a daily average file. woudl be better if it returned the values, but this works
    da.daily_avg(station, medfilter, ReqTracks, txtfile, pltit,
              extension, year1, year2, fr, csv) 
    avgf = myxdir + '/Files/' + station + '/' + txtfile
    print('Looking in ', avgf)
    x = np.loadtxt(avgf, comments='%')
    # delete the file!
    rh = x[:,2]
    medianvalue = np.median(rh);
    print('Median RH value ', medianvalue)
    # everything within 5 cm
    ii = (rh -medianvalue) < -0.05
    newx = x[ii,:] ; N=len(newx)
    if (N > 0):
        print('Suspect snow values stored in : ', snowfile)
        snow = open(snowfile, 'w+')
        for i in range(0,N):
            sv = -(newx[i,2]-medianvalue)
            print('Suspect Snow Day : ', int(newx[i,0]), int(newx[i,1]))
            snow.write("{0:4.0f}  {1:3.0f} {2:8.3f}  \n".format( newx[i,0], newx[i,1], sv))
        snow.close()
    subprocess.call(['rm','-f',avgf])
    if os.path.exists(snowfile):
        snowmask_exists = True
    else:
        print('I tried to make a snow mask file for you - but it did not work. This is most likely because ')
        print('this is a limited site, i.e. fewer tracks than the code wants. Go ahead and make one by hand,')
        print('plain text of year,doy is all you need ', snowfile)
        snowfile = None

    return snowmask_exists, snowfile

def vwc_plot(station,t_datetime, vwcdata, plot_path,circles):
    """
    makes a plot of volumetric water content

    Parameters
    ----------
    station : string
        4 ch station name

    t_datetime : datetime 
        observation times for measurements

    vwcdata : numpy array of floats (I think)
        volumetric water content

    plot_path : string
        full name of the plot file 

    circles: boolean
        circles in the plot. default is a line (really .-)

    Saves a plot to plot_path

    """
    # i think this would work???
    # data = [[1, 2, 3, 4], [0, 2, 3, 4], [0, 0 , 3, 4], [0, 0, 0, 4]] 
    # data = np.array(data)
    # data = np.where(data == 0, np.nan, data)
    for i in range(0,len(vwcdata)):
        if vwcdata[i] > 0.5:
            vwcdata[i] = np.nan
        if vwcdata[i] < 0:
            vwcdata[i] = np.nan

    plt.figure(figsize=(10, 6))
    if circles:
        plt.plot(t_datetime, vwcdata, 'bo')
    else:
        plt.plot(t_datetime, vwcdata, 'b-')
        plt.plot(t_datetime, vwcdata, 'b.')
    plt.title('GNSS-IR Soil Moisture Station ' + station.upper())
    plt.ylim(0, 0.5)
    plt.ylabel('Volumetric Soil Moisture')
    plt.grid()
    plt.gcf().autofmt_xdate()


    print(f"Saving to {plot_path}")
    plt.savefig(plot_path)

def read_apriori_rh(station,fr):
    """
    read the track dependent a priori reflector heights needed for
    phase & thus soil moisture.

    Parameters
    ----------
    station : str
        four character ID, lowercase

    fr : int
        frequency (e.g. 1,20)

    Returns
    -------
    results : numpy array 
        column 1 is just a number (1,2,3,4, etc)

        column 2 is RH in meters

        column 3 is satellite number 

        column 4 is azimuth of the track (degrees)

        column 5 is number of values used in average

        column 6 is minimum azimuth degrees for the quadrant

        column 7 is maximum azimuth degrees for the quadrant
    """
    result = []
    # do not have time to use this
    file_manager = FileManagement(station, FileTypes.apriori_rh_file)
    apriori_results = file_manager.read_file(comments='%')
    # for l2c
    myxdir = os.environ['REFL_CODE']
    apriori_path_f = myxdir + '/input/' + station + '_phaseRH.txt'

    if (fr == 1):
        apriori_path_f = myxdir + '/input/' + station + '_phaseRH_L1.txt'

    if os.path.exists(apriori_path_f):
        result = np.loadtxt(apriori_path_f, comments='%')
        print('Using: ', apriori_path_f) 
    else:
        print('Average RH file does not exist')
        sys.exit()

    # column 2 is RH, 3 is sat, 4 is azimuth, 5 is number of values
    #number_of_rhs = apriori_results[:, 4]
    #result = apriori_results
    # 50 is arbitrary - but assuming you have 365 values, not unreasonable
    # TODO make 50 a variable?
    # this is now taken care of in apriori.py
    #result = apriori_results[(number_of_rhs > 50)]


    return result


def test_func(x, a, b, rh_apriori):
    """
    This is least squares for estimating a sine wave given a fixed frequency, freqLS
    """
    freq_least_squares = 2*np.pi*2*rh_apriori/g.constants.wL2
    return a * np.sin(freq_least_squares * x + b)

def test_func_new(x, a, b, rh_apriori,freq):
    """
    This is least squares for estimating a sine wave given a fixed frequency, freqLS
    now freq is input so it is not hardwired for L2

    Parameters
    ----------
    x : numpy array of floats
        sine(elevation angle) I think
    a : float
        amplitude  - estimated
    b : float
        phase  - estimated
    rh_apriori : float
        reflector height (m)
    freq : int 
        frequency

    """
    if (freq == 20) or (freq == 2):
        freq_least_squares = 2*np.pi*2*rh_apriori/g.constants.wL2
    else:
        freq_least_squares = 2*np.pi*2*rh_apriori/g.constants.wL1

    return a * np.sin(freq_least_squares * x + b)


def phase_tracks(station, year, doy, snr_type, fr_list, e1, e2, pele, plot, screenstats, compute_lsp,gzip):
    """
    This does the main work of estimating phase and other parameters from the SNR files
    it uses tracks that were predefined by the apriori.py code

    Parameters
    ----------
    station name : string
        4 char id, lowercase
    year : int
        calendar year
    doy : int
        day of year
    snr_type : int
        SNR file extension (i.e. 99, 66 etc)
    fr_list : list of integers
        frequency, [1], [20] or [1,20] 
    e1 : float
        min elevation angle (degrees)
    e2 : float 
        max elevation angle (degrees)
    pele : list of floats 
        elevation angle limits for the polynomial removal.  units: degrees
    screenstats : bool
        whether statistics are printed to the screen
    compute_lsp : bool
        this is always true for now
    gzip : bool
        whether you want SNR files gzipped after running the code
    Only GPS frequencies are allowed because this relies on the repeating ground track.

    """

    min_amp = 3 # should be much higher - but this is primarily to catch L2P data that

    # various defaults - ones the user doesn't change in this quick Look code
    poly_v = 4 # polynomial order for the direct signal

    # this is arbitrary
    min_num_pts = 20


    # get the SNR filename
    obsfile, obsfilecmp, snrexist = g.define_and_xz_snr(station, year, doy, snr_type)

    # noise region - hardwired for normal sites ~ 2-3 meters tall
    noise_region = [0.5, 8]

    l2c_list, l5_sat = g.l2c_l5_list(year,doy)

    if not snrexist:
        print('No SNR file on this day.')
        pass

    else:
        header = "Year DOY Hour   Phase   Nv  Azimuth  Sat  Ampl emin emax  DelT aprioriRH  freq estRH  pk2noise LSPAmp\n(1)  (2)  (3)    (4)   (5)    (6)    (7)  (8)  (9)  (10)  (11)   (12)     (13)  (14)    (15)    (16)"
        file_manager = FileManagement(station, FileTypes.phase_file, year, doy, file_not_found_ok=True)
        print(f"Saving phase file to: {file_manager.get_file_path()}")
        with open(file_manager.get_file_path(), 'w') as my_file:
            np.savetxt(my_file, [], header=header, comments='%')
            # read the SNR file into memory
            sat, ele, azi, t, edot, s1, s2, s5, s6, s7, s8, snr_exists = read_snr.read_one_snr(obsfile, 1)

            for freq in fr_list:
            # read apriori reflector height results
                apriori_results = read_apriori_rh(station,freq)

                print('Analyzing Frequency ', freq, ' Year ', year, ' Day of Year ', doy)

                rows, columns = np.shape(apriori_results)

                for i in range(0, rows):
                    compute_lsp = True # will set to false for non-compliant L2C requests
                    azim = apriori_results[i, 3]
                    # this will be the satellite number you are working on 
                    sat_number = apriori_results[i, 2]
                    az1 = apriori_results[i, 5]
                    az2 = apriori_results[i, 6]
                    rh_apriori = apriori_results[i, 1]

                    x, y, nv, cf, utctime, avg_azim, avg_edot, edot2, del_t = g.window_data(s1, s2, s5, s6, s7, s8, sat, ele, azi,
                                                                                        t, edot, freq, az1, az2, e1, e2,
                                                                                        sat_number, poly_v, pele, screenstats)
                    if (freq == 20) and (sat_number not in l2c_list) :
                        if screenstats: 
                            print('Asked for L2C but this is not L2C transmitting on this day: ', int(sat_number))
                        compute_lsp = False

                    if screenstats:
                        print(f'Track {i:2.0f} Sat {sat_number:3.0f} Azimuth {azim:5.1f} RH {rh_apriori:6.2f} {nv:5.0f}')

                    if compute_lsp and (nv > min_num_pts):
                        # this is the same every track, so no reason to be in the loop
                        min_height = 0.5 ; max_height = 8 ; desired_p = 0.01

                        max_f, max_amp, emin_obs, emax_obs, rise_set, px, pz = g.strip_compute(x, y, cf, max_height,
                                                                                           desired_p, poly_v, min_height)

                        nij = pz[(px > noise_region[0]) & (px < noise_region[1])]
                        noise = 0
                        if len(nij) > 0:
                            noise = np.mean(nij)
                            obs_pk2noise = max_amp/noise

                            if screenstats:
                                print(f'>> LSP RH {max_f:7.3f} m {obs_pk2noise:6.1f} Amp {max_amp:6.1f} {min_amp:6.1f} ')
                        else:
                            max_amp = 0  # so it will have a value

                    # http://scipy-lectures.org/intro/scipy/auto_examples/plot_curve_fit.html
                        if (nv > min_num_pts) and (max_amp > min_amp):
                            minmax = np.max(x) - np.min(x)
                            if (minmax > 22) and (del_t < 120):
                                x_data = np.sin(np.deg2rad(x))  # calculate sine(E)
                                y_data = y
                                test_function_apriori = partial(test_func_new, rh_apriori=rh_apriori,freq=freq)
                                #test_function_apriori = partial(test_func, rh_apriori=rh_apriori)
                                params, params_covariance = optimize.curve_fit(test_function_apriori, x_data, y_data, p0=[2, 2])
    

                                # change phase to degrees
                                phase = params[1]*180/np.pi
                                # calculate min and max elevation angle
                                min_el = min(x); max_el = max(x)
                                amp = np.absolute(params[0])
                                raw_amp = params[0]
                                if phase > 360:
                                    phase = phase - 360
                                    if phase > 360:
                                        phase = phase - 360

                                # do not allow negative amplitudes. 
                                if raw_amp < 0:
                                    phase = phase + 180

                                result = [[year, doy, utctime, phase, nv, avg_azim, sat_number, amp, min_el, max_el, del_t, rh_apriori, freq, max_f, obs_pk2noise, max_amp]]
                                np.savetxt(my_file, result, fmt="%4.0f %3.0f %6.2f %8.3f %5.0f %6.1f %3.0f %5.2f %5.2f %5.2f %6.2f %5.3f %2.0f %6.3f %6.2f %6.2f", comments="%")
        # gzip SNR file if requested
        if gzip:
            subprocess.call(['gzip', obsfile])


def low_pct(amp, basepercent):
    """
    emulated amp_normK code from PBO H2O
    inputs are the amplitudes  and a percentage
    used to define the bottom level. returns
    normalized amplitudes

    this is meant to be used by individual tracks (I think)
    in this case they are the top values, not the bottom ... ugh
    """
    n = len(amp)
    if n > 0:
        numx = int(np.round(n*basepercent) )
        if numx == 0:
            numx = n
        sort_a = np.sort(amp)
        lowval = np.mean(sort_a[0:numx])

    return lowval


def convert_phase(station, year, year_end=None, plt2screen=True,fr=20,tmin=0.05,tmax=0.5,polyorder=-99,circles=False,
        subdir='',hires_figs=False):
    """
    Convert GPS phase to VWC. Using Clara Chew's algorithm from 
    Matlab write_vegcorrect_smc.m

    https://scipy-cookbook.readthedocs.io/items/SignalSmooth.html

    Parameters
    -----------
    station : str
        4 char station name
    year : int
        beginning year
    year_end : int
        last year
    plt2screen : boolean
        plots come to the screen
    fr : integer
        frequency
        default is L2C (20)
    tmin : float
        soil texture minimum
    tmax : float
        soil texture maximum
    polyorder : integer
        override on the polynomial order used in leveling
    circles : boolean
        final plot using circles (instead of line)
    subdir : str
        subdirectory for $REFL_CODE/Files
    hires_figs : bool
        whether you want eps instead of png files created

    """

    if not year_end:
        year_end = year

    # read makejson
    station_file = FileManagement(station, 'make_json')
    json_data = station_file.read_file()

    if json_data['lat'] >= 0:
        print('Northern hemisphere summer')
        southern = False
    elif json_data['lat'] < 0:
        print('Southern hemisphere summer')
        southern = True

    else:
        print(f"the required json file created by gnssir_input could not be found: {station_file.get_file_path()}")
        sys.exit()

    # for PBO H2O this was set using STATSGO. 5% is reasonable as a starting point for australia
    #tmin = 0.05  # for now
    #print('minimum texture value', tmin)
    residval = 2  # for now

    # daily average file of phase results
    #file_manager = FileManagement(station, FileTypes.daily_avg_phase_results)
    #avg_phase_results = file_manager.read_file(comments='%')

    myxdir = os.environ['REFL_CODE']


    # begging for a function ...
    # overriding Kelly's code for now
    if (fr == 1):
        fileout = myxdir + '/Files/' + subdir + '/' + station + '_phase_L1.txt'
    else:
        fileout = myxdir + '/Files/' + subdir + '/' + station + '_phase.txt'
    print(subdir)

    if os.path.exists(fileout):
        avg_phase_results = np.loadtxt(fileout, comments='%')
    else:
        print('Average phase results not found')
        sys.exit()
    if (len(avg_phase_results) == 0):
        print('Empty results file')
        sys.exit()


    # get only the years requested - this matters if in previous steps more data was processed
    avg_phase_results_requested = np.array([v for v in avg_phase_results if v[0] in np.arange(year, year_end+1)])

    years = avg_phase_results_requested[:, 0]
    doys = avg_phase_results_requested[:, 1]
    ph = avg_phase_results_requested[:, 2]
    phsig = avg_phase_results_requested[:, 3] # TODO this is not used in the rest of the code
    amp = avg_phase_results_requested[:, 4]
    months = avg_phase_results_requested[:, 6]
    days = avg_phase_results_requested[:, 7]

    t = years + doys/365.25
    tspan = t[-1] - t[0]
    print('>>> Timespan in years: ', np.round(tspan,3))
    if tspan < 1.5:
        polyordernum = 0  # do nothing
    else:
        polyordernum =  int(np.floor(tspan - 0.5));
    print('>>> Polynomial Order ', polyordernum)

    # need to do a 30 day smoother on amp
    window_len = 30
    s = np.r_[amp[window_len - 1:0:-1], amp, amp[-2:-window_len - 1:-1]]
    w = np.ones(window_len, 'd')
    y = np.convolve(w / w.sum(), s, mode='valid')
    # NOTE: length(output) != length(input), to correct this: return y[(window_len/2-1):-(window_len/2)] instead of just y.
    y0 = int(window_len / 2 - 1);
    y1 = -int(window_len / 2)
    smamp = y[y0:y1]

    # TODO do we want this plot as an option to display?
    # plt.figure() #plt.plot(t, smamp, '-', t, amp,'o') #plt.show()

    # original clara chew code
    # residval is the median of the lowest 10 % of the values
    wetwt = 10.6 * np.power(smamp, 4) - 34.9 * np.power(smamp, 3) + 41.8 * np.power(smamp, 2) - 22.6 * smamp + 5.24

    # TODO this variable (below) is never used in the code
    delphi = -5.65 * np.power(wetwt, 4) + 43.9 * np.power(wetwt, 3) - 101 * np.power(wetwt,
                                                                                     2) + 20.4 * wetwt - 2.37;
    # I think this is just changing the units by factor of 100
    delphi = (smamp - 1) * 50.25 / 1.48;  # % CR 17jan12
    newph = ph - delphi;
    # resid value should be min value of the actual data

    vwc = 100 * tmin + 1.48 * (ph - residval);
    # this is our "vegetation corrected" value - though needs leveling
    newvwc = 100 * tmin + 1.48 * (newph - residval);
    # acting on newvwc, year and doys
    nodes = np.empty(shape=[0, 3])

    if southern:
        for yr in np.arange(year, year_end+1):
            y1 = yr - 30/365.25  # dec previous year
            y2 = yr + 45/365.25  # mid feb
            print(yr, y1, y2)  # kristine's idea of summer in australia
            # summer_vwc = newvwc[(year == y) &   (doys >= 1) & (doys < 40)]
            summer_vwc = newvwc[(t > y1) & (t < y2)]
            # summer_vwc2 = newvwc[(year == (y-1))  &   (doys >= 334) & (doys < 365)]
            # new_summer = np.vstack((summer_vwc, summer_vwc2))
            if len(summer_vwc) > 0:
                basepercent = 0.15
                lval = low_pct(summer_vwc, basepercent)
                newl = [yr, 15, lval]
                #print(newl)
                nodes = np.append(nodes, [newl], axis=0)
                print('Level value:', yr, np.round(lval,2))
            else:
                print('No summer dates found to compute VWC for ', yr)
    else:
        # northern hemisphere
        for yr in np.arange(year, year_end+1):
            # put in amplitude criteria to keep out bad L2P results
            summer_vwc = newvwc[(years == yr) & (doys >= 190) & (doys < 273)]
            if len(summer_vwc) > 0:
                basepercent = 0.15
                lval = low_pct(summer_vwc, basepercent)
                newl = [yr, 213, lval]
                #print(newl)
                nodes = np.append(nodes, [newl], axis=0)
                print('Level value:',yr, np.round(lval,2))
            else:
                print('No summer dates found to compute VWC', yr)


    plt.figure(figsize=(10, 10))
    plt.subplots_adjust(hspace=0.2)
    plt.suptitle(f'Station: {station}', size=16)

    # list comprehension to make datetime objects from year, doy
    t_datetime = [datetime.strptime(f'{int(yr)} {int(d)}', '%Y %j') for yr, d in zip(years, doys)]

    # create subplots: 2 rows 1 column, 1st subplot
    ax = plt.subplot(2, 1, 1)

    ax.plot(t_datetime, ph, 'b-',label='original')
    ax.plot(t_datetime, newph, 'r-',label='vegcorrected')
#    ax.plot(t_datetime, ph-newph, 'm-',label='subtracted')
    ax.set_title(f'With and Without Vegetation Correction ')
    ax.set_ylabel('phase (degrees)')
    ax.legend(loc='best')
    ax.grid()
    # ?? does not seem to be working.  sigh
    plt.gcf().autofmt_xdate()


    # More descriptive variable names would help 
    st = nodes[:, 0] + nodes[:, 1]/365.25
    st_datetime = [datetime.strptime(f'{int(yr)} {int(d)}', '%Y %j') for yr, d in zip(nodes[:, 0], nodes[:, 1])]
    sp = nodes[:, 2]

    howmanynodes = len(sp)
    print('number of nodes', howmanynodes)
    if howmanynodes == 0:
        print('No summer nodes found. Exiting.')
        if plt2screen:
            plt.show()
        sys.exit()
    
    else:
        polyordernum = howmanynodes - 1
    if polyorder >= 0: # default is -99
        print('>>> Using user override for polynomial order on leveling')
        polyordernum = polyorder

    anothermodel = np.polyfit(st, sp, polyordernum)
    new_level = np.polyval(anothermodel, t)

    # this is applying the level and the tmin values
    nv = tmin + (newvwc - new_level) / 100

    ax = plt.subplot(2, 1, 2)
    ax.plot(t_datetime, newvwc, 'b-', label='new vwc')

    ax.plot(st_datetime, sp, 'ro', label='nodes')
    ax.plot(t_datetime, new_level, 'r-', label='level')
    ax.set_ylabel('VWC')
    ax.set_title('Volumetric Water Content')
    ax.legend(loc='best')
    ax.grid()
    plt.gcf().autofmt_xdate()

    outdir = f'{xdir}/Files/{subdir}'

    if hires_figs:
        plot_path = f'{outdir}/{station}_phase_vwc_result.eps'
    else:
        plot_path = f'{outdir}/{station}_phase_vwc_result.png'
    print(f"Saving to {plot_path}")
    plt.savefig(plot_path)


    if hires_figs:
        plot_path = f'{outdir}/{station}_vol_soil_moisture.eps'
    else:
        plot_path = f'{outdir}/{station}_vol_soil_moisture.png'

    vwc_plot(station,t_datetime, nv, plot_path,circles) 

    if plt2screen:
        plt.show()

    vwcfile = FileManagement(station, FileTypes.volumetric_water_content).get_file_path()

    vwcfile = f'{outdir}/{station}_vwc.txt'
    print('>>> VWC results being written to ', vwcfile)
    with open(vwcfile, 'w') as w:
        N = len(nv)
        w.write("% Soil Moisture Results for GNSS Station {0:4s} \n".format(station))
        w.write("% {0:s} \n".format('https://github.com/kristinemlarson/gnssrefl'))
        w.write("% FracYr    Year   DOY   VWC Month Day \n")
        for iw in range(0, N):
            whydoys = np.round(365.25 * (t[iw] - years))

            fdate = t[iw]
            myyear = years[iw]
            mm = months[iw]
            dd = days[iw]
            mydoy = doys[iw]
            watercontent = nv[iw]
            # we do not allow negative soil moisture in my world.
            if (watercontent> 0 and watercontent < 0.5):
                w.write(f"{fdate:10.4f} {myyear:4.0f} {mydoy:4.0f} {watercontent:8.3f} {mm:3.0f} {dd:3.0f} \n")


def write_avg_phase(station, phase, fr,year,year_end,minvalperday,vxyz,subdir):

    """
    creates output file for average phase results

    Parameters
    ----------
    station : string

    phase : numpy list  (float)
         phase values 

    fr : int
        frequency

    year: int
        first year evaluated

    year_end : int
        last year evaluated

    minvalperday : int
        required number of satellite tracks to trust the daily average 

    vxyz is from some other compilation

    subdir : str
        subdirectory for results

    Returns
    -------
    tv : numpy array with elements
        year 
        doy  - day of year
        meanph - mean phase value in degrees
        nvals - number of values that went into the average

    """
    myxdir = os.environ['REFL_CODE']
    y1 = vxyz[:, 0]
    d1 = vxyz[:, 1]
    phase = vxyz[:, 2]
    sat = vxyz[:, 3] # this is not used
    az = vxyz[:, 4] # this is not used
    rh = vxyz[:, 5] # this is not used
    amp = vxyz[:, 6]

    tv = np.empty(shape=[0, 4])

    # ultimately would like to use kelly's code here
    if (fr == 1):
        fileout = myxdir + '/Files/'  + subdir + '/' + station + '_phase_L1.txt'
    else:
        fileout = myxdir + '/Files/' + subdir + '/' + station + '_phase.txt'

    print('Daily averaged phases will be written to : ', fileout)
    with open(fileout, 'w') as fout:
        # Year DOY Ph Phsig NormA MM DD
          #            2012   1  10.00   2.60  0.962  0.00    1  1
        fout.write("% Year DOY   Ph    Phsig NormA  empty  Mon Day \n")
        for requested_year in range(year, year_end + 1):
            for doy in range(1, 367):
            # put in amplitude criteria to keep out bad L2P results
                ph1 = phase[(y1 == requested_year) & (d1 == doy) & (phase > -10) & (amp > 0.65)]
                amp1 = amp[(y1 == requested_year) & (d1 == doy) & (phase > -10) & (amp > 0.65)]
                if (len(ph1) > minvalperday):
                    newl = [requested_year, doy, np.mean(ph1), len(ph1)]
                # i think you normalize the individual satellites before this step
                #namp = qp.normAmp(amp1,0.15)
                    tv = np.append(tv, [newl], axis=0)
                    rph1 = np.round(np.mean(ph1), 2)
                    meanA = np.mean(amp1)
                    rph1_std = np.std(ph1)
                    yy, mm, dd, cyyyy, cdoy, YMD = g.ydoy2useful(requested_year, doy)
                    fout.write(f" {requested_year:4.0f} {doy:3.0f} {rph1:6.2f} {rph1_std:6.2f} {meanA:6.3f} {0.0:5.2f}   {mm:2.0f} {dd:2.0f} \n")

        fout.close()
    return tv

def apriori_file_exist(station,fr):
    """
    reads in the a priori RH results

    Parameters
    ----------
    station : string
        station name

    fr : integer
        frequency
        
    Returns
    -------
    boolean as to whether the apriori file exists

    """
    # do not have time to use this
    file_manager = FileManagement(station, FileTypes.apriori_rh_file)
    # for l2c
    myxdir = os.environ['REFL_CODE']
    apriori_path_f = myxdir + '/input/' + station + '_phaseRH.txt'

    if (fr == 1):
        apriori_path_f = myxdir + '/input/' + station + '_phaseRH_L1.txt'
    
    return os.path.exists(apriori_path_f) 

def load_phase_filter_out_snow(station, year1, year2, fr,snowmask):
    """
    Load all phase data and attempt to remove outliers from snow if snowmask provided. 

    Parameters
    ----------
    station : str
        four character station name

    year1 : int
        starting year
    year2 : int
        ending year
    fr : int
        frequency, i.e. 1 or 20
    snowmask : str
        name/location of the snow mask file
        None if this value is not going to be used

    Returns
    -------
    dataexist : bool
        whether phase data were found
    year : numpy array of int
        calendar years
    doy : numpy array of int
        day of year
    hr : numpy array of floats
        UTC hour of measurement
    ph : numpy array of floats
        LS phase estimates
    azdata : numpy array of floats
        average azimuth, degrees
    ssat  : numpy array of int
        satellite number
    rh : numpy array of floats
        reflector height, meters
    amp_lsp : numpy array of floats
        lomb scargle periodogram amplitude
    amp_ls : numpy array of floats
        least squares amplitude
    ap_rh : numpy array of floats
        apriori rh
    results_trans : numpy array
        all phase results concatenated into numpy array
        plus column for quadrant and unwrapped phase

    """
#   now load the phase data
    newresults = []
    results_trans = []
    vquad = np.empty(shape=[0, 16])
    xdir = os.environ['REFL_CODE']
    # output will go to 
    fname = xdir + '/Files/' + station + '/raw.phase'  

    dataexist, results = load_sat_phase(station, year1,year2, fr)
    results = results.T # backwards for consistency
    if snowmask == None:
        nr,nc = np.shape(results)
        if nr > 0:
            print('Number of rows and columns ', nr,nc)
            raw = write_out_raw_phase(results,fname)
    else:
        print('Snow mask should exist')
        override = np.loadtxt(snowmask, comments='%')
        # adding nonsense because it doesn't like having only one value
        #  which is a kluge i know
        blah = np.array([[1, 2], [3, 4]])
        override = np.vstack((override, blah))
        for year in range(year1,year2+1):
            ii = (results[:,0] == year) & (results[:,12] == fr)
        # it is easier for me to do this year by year
            y = results[ii,0];
            d = results[ii,1]
            ph = results[ii,3]
            thisyear_results = results[ii,:]
        # the goal is to find all the results that do not appear in the override list
            jj = (override[:,0] == year) # find days when there are suspects points
            targetdoys = override[jj,1]
            #print(year, ' length of d', len(d))
            # the mask asks when is d distinct from targetdoys
            mask=np.logical_not(np.isin(d,targetdoys))
            #print(year, ' length of dmasked', len(d[mask]))
            masked_results = thisyear_results[mask]
            newresults = np.append(newresults, masked_results)
            #vquad = np.vstack((vquad, thisyear_results[mask] ) )
            vquad = np.append(vquad, masked_results,axis=0)
    #  
        nr,nc = np.shape(vquad)
        if nr > 0:
            print('Number of rows and columns after snow filter ', nr,nc)
            raw = write_out_raw_phase(vquad,fname)

    rtrans= np.transpose(raw)

    if len(rtrans) == 0:
        print('no results ')
        dataexist = False
        year = []; doy = []; hr = []; ph = []; azdata = []; ssat = []
        rh = []; amp_lsp = []; amp_ls=[]; ap_rh = []
    else:
        dataexist = True
        # save with new variable names
        year = rtrans[0]
        doy = rtrans[1]
        hr = rtrans[2]
        ph = rtrans[3]
        azdata = rtrans[5]
        ssat = rtrans[6]
        amp_ls = rtrans[7]
        ap_rh = rtrans[11]
        rh = rtrans[13]
        amp_lsp = rtrans[15]

    return dataexist, year, doy, hr, ph, azdata, ssat, rh, amp_lsp, amp_ls, ap_rh, rtrans


def help_debug(rt,xdir, station):
    """
    Takes the input of phase files, read by other functions, and writes out a 
    file to help with debugging  (comparion of matlab and python codes)

    rt : numpy array of floats
        contents of the phase files stored in a numpy array 
    xdir : str
        where the otuput should be written
    station : str
        name of the station

    """
    fname = xdir + '/Files/' + station +'/tmp.' + station
    # don't have a priori rh values at this point
    rhtrack = 0
    #if True:
        # open the file
        #debug = write_all_phase(rt,fname,None,1,rhtrack)
        # write to the file
    #    debug = write_all_phase(rt,fname,debug,2,rhtrack)
    #    debug.close()


def load_avg_phase(station,fr):
    """
    loads a previously computed daily average phase solution.
    this is NOT the same as the multi-track phase results.
    This file is now stored in station subdirectory in $REFL_CODE/Files/

    Parameters
    ----------
    station : str
        4 character station ID, lowercase
    fr : int
        frequency

    Returns
    -------
    avg_exist : bool
        whether the necessary file exists
    avg_date : list of floats
        fractional year, i.e. year + doy/365.25
    avg_phase : list of floats
        average phase for a given day

    """

    avg_date = []
    avg_phase = []
    avg_exist = False

    if fr == 1:
        xfile = f'{xdir}/Files/{station}/{station}_phase_L1.txt'
    else:
        xfile = f'{xdir}/Files/{station}/{station}_phase.txt'

    if os.path.exists(xfile):
        result = np.loadtxt(xfile, comments='%')
        if len(result) > 0:
            year = result[:, 0]
            day = result[:, 1].astype(int)
            phase = result[:, 2]
            # change to integers from float, though not necessary since i'm using float date...
            avg_date = year + day/365.25
            avg_phase = phase
            avg_exist = True

    if not avg_exist:
        print('WARNING The average phase file used from a previous run for QC does not exist as yet')

    return avg_exist, avg_date, avg_phase


def load_sat_phase(station, year, year_end, freq):
    """
    Picks up the phase estimates from local (REFL_CODE) results section
    and returns most of the information from those files

    Parameters
    ----------
    station : str
        four character station name
    year : integer
        beginning year
    year_end : integer
        ending year
    freq : integer
        GPS frequency (1,20 allowed)

    Returns
    -------
    dataexist : bool
        whether data found?
    results : numpy array of floats
        basically one variable with everything in the original columns from the daily phase files

    """
    print('Requested frequency: ', freq)
    dataexist = False
    xdir = os.environ['REFL_CODE']
    xfile = xdir + '/input/override/' + station + '_vwc' 
    found_override = False
    # not implementing this yet
    if os.path.exists(xfile):
        print('found override file but not implementing at this time')
    #    override = np.loadtxt(xfile, comments='%')
    #    found_override = True

    thedir = Path(os.environ["REFL_CODE"])

    if not year_end:
        year_end = year

    results = []
    for yyyy in range(year, year_end+1):
        print('reading in year', yyyy)
        # where the results are stored
        data_dir = thedir / str(yyyy) / 'phase' / station
        local_results = read_files_in_dir(data_dir)
        if local_results:
            results.extend(local_results)

    if not results:
        print(f"No results were found for the year range you requested: ({year}-{year_end})")
        sys.exit()

    results = np.array(results)
    freq_list = results[:, 12]
    ii = (freq_list == freq)
    results = results[ii, :]
    print('Total phase measurements for this frequency: ', len(results))
#    common_elements, ar1_i, ar2_i = np.intersect1d(ar1, ar2, return_indices=True)
    #minyear = np.min(np.unique(results[:,0]))
    #maxyear = np.max(np.unique(results[:,0]))
    nr,nc = np.shape(results)
    print(nr, ' number of rows')

    results = results.T  # dumb, but i was using this convention.  easier to maintain

    if len(results) > 0:
        dataexist = True 

    return dataexist, results


def set_parameters(station, minvalperday,tmin,tmax,min_req_pts_track,fr, year, year_end,subdir,plt,auto_removal,warning_value):
    """

    Parameters
    ----------
    station : str
        4 character station name

    Returns
    -------
    minvalperday : int
        number of phase values required each day
    tmin : float
        min soil texture
    tmax : float
        max soil texture
    min_req_pts_track: int 
        minimum number of phase values per year per track
    freq : int
        frequency to use (1,20 allowed)
    year_end : int
        last year to analyze
    subdir : str
        name for subdirectory used in subdirectory of REFL_CODE/Files
    plt : bool
        whether you want plots to come to the screen
    auto_removal : bool
        whther tracks should be removed when they fail QC
    warning_value : float
        phase RMS needed to trigger warning
    plot_legend : bool
        whether to plot PRN numbers on the phase & amplitude results

    """
    # originally this was for command line interface ... 
    remove_bad_tracks = auto_removal # ??
    plot_legend = True # default is to use them
    circles = False

    g.checkFiles(station, '')

    # not using extension
    lsp = gnssir.read_json_file(station, '')
    # pick up values in json, if available
    if 'vwc_min_soil_texture' in lsp:
        tmin = lsp['vwc_min_soil_texture']
    if 'vwc_max_soil_texture' in lsp:
        tmax = lsp['vwc_max_soil_texture']
    if 'vwc_minvalperday' in lsp:
        minvalperday = lsp['vwc_minvalperday']
    if 'vwc_warning_value' in lsp:
        warning_value = lsp['vwc_warning_value']
    if 'vwc_min_req_pts_track' in lsp:
        min_req_pts_track = lsp['vwc_min_req_pts_track']
    if 'vwc_plot_legend' in lsp:
        plot_legend = lsp['vwc_plot_legend']
    if 'vwc_circles' in lsp:
        circles = lsp['vwc_circles']
    if 'vwc_min_norm_amp' in lsp:
        min_norm_amp = lsp['vwc_min_norm_amp']
    else:
        min_norm_amp = 0.5  # trying this out

    if (len(station) != 4):
        print('station name must be four characters')
        sys.exit()

    if (len(str(year)) != 4):
        print('Year must be four characters')
        sys.exit()

    freq = fr # KE kept the other variable

    if not year_end:
        year_end = year 

    # originally was making people input these on the command line
    # now first try to read from json.  if not there they will be set here

    if tmin is None:
        tmin = 0.05

    if tmax is None:
        tmax = 0.5

    # default is station name
    if subdir == None:
        subdir = station 

    # make sure subdirectory exists
    g.set_subdir(subdir)

    if not plt:
        print('no plots will come to screen. Will only be saved.')

    print('minvalperday/tmin/tmax/min_req_tracks', minvalperday, tmin, tmax, min_req_pts_track)

    return minvalperday, tmin, tmax, min_req_pts_track, freq, year_end, subdir, \
            plt, remove_bad_tracks, warning_value, min_norm_amp, plot_legend,circles

def write_all_phase(v,fname):
    """
    writes out preliminary phase values and other metrics for advanced vegetation
    option.  This is in the hope that it can be used in clara chew's
    dissertation algorithm.

    File is written to $REFL_CODE/Files/station/station_all_phase.txt I think

    Parameters
    ----------
    v : numpy of floats as defined in vwc_cl
        TBD
        year, doy, phase, azimuth, satellite number
        estimated RH, LSP amplitude, LS amplitude, UTC hours
        raw LSP amp, raw LS amp
    fname : str
        name of the output file
    filestatus : int
        1, open the file
        2, write to file (well, really any value)
    rhtrack: float
        apriori reflector height for the given track, meters

    Returns
    -------
    allrh : fileID

    """

    #if filestatus == 1:
    if True:
        print('Writing interim phase values to ', fname)
        header1=  'Year DOY Hour   Phase   Nv  Azimuth  Sat  Ampl emin emax  DelT aprioriRH  freq estRH  pk2noise LSPAmp'
        header2 = '(1)  (2)  (3)    (4)   (5)    (6)    (7)  (8)  (9)  (10)  (11)   (12)     (13)  (14)    (15)    (16)'

        #allrh.write(" {0:s}  \n".format(header1))
        #allrh.write(" {0:s}  \n".format(header2))

        #return allrh

    nr,nc = v.shape
    # sort by time
    ii = np.argsort(v[:,0] + v[:,1]/365.25)
    v = v[ii,:]

    with open(fname, 'w') as my_file:
        np.savetxt(my_file, [], header=header1, comments='%')
        #np.savetxt(my_file, [], header=header2, comments='%')
    #result = [[year, doy, utctime, phase, nv, avg_azim, sat_number, amp, min_el, max_el, del_t, rh_apriori, freq, max_f, obs_pk2noise, max_amp]]
    #np.savetxt(my_file, result, fmt="%4.0f %3.0f %6.2f %8.3f %5.0f %6.1f %3.0f %5.2f %5.2f %5.2f %6.2f %5.3f %2.0f %6.3f %6.2f %6.2f", comments="%")


    #for i in range(0,nr):
    #    q = old_quad(v[i,3])
    #    a = v[i,5] ; sat = v[i,6]; amp_lsp = v[i,15]
    #    amp_ls = v[i,7] ; rh = v[i,13]
    #    allrh.write(" {0:4.0f} {1:3.0f} {2:7.3f} {3:7.1f} {4:2.0f} {5:7.3f} {6:7.3f} {7:7.3f} {8:2.0f} \
    #            {9:7.2f} {10:7.2f} {11:7.2f} {12:7.3f}\n".format(v[i,0], v[i,1], v[i,3],a, sat,rh,amp_lsp,amp_ls,\
    #            q, v[i,2],v[i,10], rhtrack ))


def old_quad(azim):
    """
    calculates oldstyle quadrants from PBO H2O

    Parameters
    ----------
    azim : float
        azimuth, dgrees
    q : int
        old quadrant system used in pboh2o
    """
    q = 1
    if azim < 90:
        q= 1
    elif (azim >= 90) and (azim < 180):
        q = 4 
    elif (azim >= 180) and (azim < 270):
        q = 3
    elif (azim >= 270) and (azim <= 360):
        q =2

    return q


def kinda_qc(satellite, rhtrack,meanaztrack,nvalstrack, amin,amax, y, t, new_phase, 
             avg_date,avg_phase,warning_value,ftmp,remove_bad_tracks,k4,avg_exist):
    """
    Parameters
    ----------
    satellite : int
        satellite number
    rhtrack: float
        a priori reflector height 
    meanaztrack : float
        I think it is the azimuth of the track, degrees
    nvalstrack : int
        not sure?
    amin : int
        min az of this quadrant
    amax : int
        max az of this quadrant
    y : numpy array of ints
        year
    t : numpy array of ints
        day of year
    new_phase : numpy array of floats
        phase values for a given satellite track ??
    avg_date : numpy array of floats
        y + doy/365.25 I think
    avg_phase : numpy array of floats
        average phase, in degrees
    warning_value : float
        phase noise value
    ftmp : file ID 
        for writing
    remove_bad_tracks : bool
        whether you write out new tracks with bad ones removed
    k4 : int
        number of tracks?
    avg_exist : bool
        whether you have previous solution to compare to
    
        
    """
    # this is a kind of quality control -use previous solution to have 
    # better feel for whether current solution works. defintely needs to go in a function
    if avg_exist:
        # quadrant results for this satellite track
        satdate = y + t/365.25
        satphase = new_phase
        keepit=False

        # figure out intersection with "good" results
        inter, id1, id2 = np.intersect1d(avg_date, satdate, assume_unique=True, return_indices=True)
        aa = avg_phase[id1]
        bb = satphase[id2]
        if len(aa) > 0:
            res = np.round(np.std(aa - bb), 2)
            addit = ''
            keepit = True
            if (res > warning_value ) :
                addit = '>>>>>  Consider Removing This Track <<<<<'
                if remove_bad_tracks:
                    addit = '>>>>>  Removing This Track - rerun to see effect <<<<<'
                    keepit = False
            if keepit:
                ftmp.write("{0:3.0f} {1:7.2f} {2:3.0f} {3:7.1f} {4:7.0f} {5:4.0f} {6:4.0f} \n".format(k4,rhtrack, 
                       satellite,meanaztrack,nvalstrack,amin,amax))
                k4 = k4 + 1
                print(f"Npts {len(aa):4.0f} SatNu {satellite:2.0f} Residual {res:6.2f} Azims {amin:3.0f} {amax:3.0f} {addit:20s} ")

    else:
        print('You do not have a previous solution to compare to so I cannot compute QC stats. Rerun vwc')

    return k4

def save_vwc_plot(fig, pngfile):
    """
    Parameters
    ----------
    fig : matplotlib figure
        the figure definition you define when you open a figure
    pngfile : str
        name of the png file to be saved
    """
    fig.savefig(pngfile,format="png")
    print('Saving to ', pngfile)

    return

def rename_vals(year_sat_phase, doy, hr, phase, azdata, ssat, amp_lsp, amp_ls, rh, ap_rh,ii):
    """
    this is just trying to clean up vwc.py  
    send indices ii - and return renamed variables.  

    Parameters
    ----------
    year_sat_sat
    doy : 
    hr : 
    phase : 
    azdata : 
    ssat : 
    amp_lsp : 
    amp_ls
    rh : 
    ap_rh :
    ii :

    Returns
    -------
    y : numpy array of int
        year
    t : numpy array of int
        day of year 
    h : numpy array of floats
        hour of the day (UTC)
    x : numpy array of floats
        phase, degrees
    azd: numpy array of floats
        azimuth for the track
    s : numpy array of int
    amps_lsp : numpy array of floats
        LSP amplitude
    amps_ls : numpy array of floats
        least squares amplitude
    rhs : numpy array of floats
        estimated RH (m)
    ap_rhs : numpy array of floats
        a priori RH (m)

    """
    y = year_sat_phase[ii]
    t = doy[ii]
    h = hr[ii] # this is fractional hour of the day, GPS time 
    x = phase[ii]
    # should be so we could do subdaily VWC
    azd = azdata[ii] # azimuth, in degrees
    s = ssat[ii] # array of satellites numbers
    amps_lsp = amp_lsp[ii] # this amplitude is RH amplitude 
    amps_ls = amp_ls[ii] # this amplitude is phase amplitude 
    rhs = rh[ii] # estimated RH
    ap_rhs = ap_rh[ii] # apriori RH

    return y,t,h,x,azd,s,amps_lsp,amps_ls,rhs, ap_rhs


def write_out_raw_phase(v,fname):
    """
    write daily phase values used in vwc to a new consolidated file
    I added columns for quadrant and unwrapped phase

    Parameters
    ----------
    v : numpy array
        phase results read for multiple years. 
        could be with snow filter applied
    fname : str
        filename for output

    Returns
    -------
    newv : numpy array
        original variable v with columns added for
        quadrant (1-4) and unwrapped phase 

    """

    print('Writing to ', fname)
    # make sure v is numpy 
    v = np.asarray(v)
    nr,nc = v.shape; 
# sort by time
    ii = np.argsort(v[:,0] + v[:,1]/365.25)
    v = v[ii,:]
# calculate quadrants cause it makes life easier
    q = np.zeros((nr,1)) #
    for i in range(nr):
        q[i] = old_quad(v[i,5])

    # make a column full of zeros to store unwrapped phase
    unwrapped = np.zeros((nr,1)) #

    # add quadrant column
    newv = np.hstack((v,q))
    # add unwrapped column
    newv = np.hstack((newv,unwrapped))

    # get a list of possible satellites
    sat_list = np.unique(v[:,6])
    # discontinuity used in phase wrapping
    discont = 270

    # make a variable to return
    tnewv = np.zeros((nr,18)) #
    for ql in [1,2,3,4]:
        for s in sat_list:
            iw = (newv[:,16] == ql) & (newv[:,6] == s)
            tnewv = newv[iw,:]
            if (len(tnewv) > 0):
                doy = tnewv[:,1]; phase = tnewv[:,3]
                unwrap_phase = np.unwrap(phase,period=360,discont=discont)
                newv[iw,17] = unwrap_phase

    # headers for output file
    h1 = "Year DOY Hour   Phase   Nv  Azim   Sat  Ampl emin emax   DelT aprRH  fr  estRH  pk2n  LSPAmp  quad  unphase\n"   
    h2 = "(1)  (2)  (3)    (4)   (5)   (6)   (7)  (8)  (9)  (10)  (11)   (12)  (13)  (14)   (15)   (16)  (17)  (18) "

    with open(fname, 'w') as my_file:
        np.savetxt(my_file, newv, fmt="%4.0f %3.0f %6.2f %8.3f %5.0f %6.1f %3.0f %5.2f %5.2f %5.2f %6.2f %5.3f %2.0f %6.3f %6.2f %6.2f %2.0f %8.3f ",header=h1+h2,comments='%')

    return newv

def write_phase_for_advanced(filename, vxyz):
    """
    Writes out a file of interim phase results for advanced models
    developed by Clara Chew

    File generally written to $REFL_CODE/Files/<station>/all_phase.txt

    Parameters
    ----------
    filename : str
        name for output file
    vxyz : numpy array of floats
        as defined in vwc_cl.py
    """
    # do a quick sort as they are likely quadrant sorted now
    ii = np.argsort(vxyz[:,0] + vxyz[:,1]/365.25)
    vxyz = vxyz[ii,:]
    #headers for output file - these are not correct BTW
    print('writing interim file for advanced model ', filename)
    h1 = "Year DOY Phase  Azim Sat    RH    nLSPA nLSA    Hour   LSPA   LS  apRH  quad  delRH  vegM\n"
    h2 = "(1)  (2)  (3)   (4)  (5)    (6)    (7)   (8)     (9)   (10)  (11)  (12)  (13)  (14)  (15) "
    #2012   1   9.19  315.6  1   1.850   0.97   0.98  0.59 19.80 19.79  1.85  2
    with open(filename, 'w') as my_file:
        np.savetxt(my_file, vxyz, fmt="%4.0f %3.0f %6.2f %6.1f %2.0f %7.3f %6.2f %6.2f %5.2f %6.2f %6.2f %6.3f %2.0f %6.3f %2.0f  ",header=h1+h2,comments='%')

    return
#                   if adhoc_snow:
#                        ii = (norm_ampLSP > 0.5)
#                        y,t,h,new_phase,azd,s,amp_lsps,amp_lss,rhs,ap_rhs = \
#                                qp.rename_vals(y, t, h, new_phase, azd, s, amp_lsps, amp_lss, rhs, ap_rhs,ii)
#                        norm_ampLSP = norm_ampLSP[ii]
#                        norm_ampLS = norm_ampLS[ii]
#                        fracyear = fracyear[ii]




import argparse
import datetime
import numpy as np
import os
import requests
import subprocess
import sys
import gnssrefl.gps as g


def download_ioc(station: str, date1: str, date2: str, output: str = None, plt: bool = False, outliers: bool = False, sensor= None, subdir: str=None):
    """
    Downloads and saves IOC tide gauge files

    Parameters
    ----------
    station : str
        IOC station name 

    date1 : str
        begin date in yyyymmdd.
        Example value: 20150101

    date2 : str
        end date in yyyymmdd.
        Example value: 20150101

    output : str 
        Optional output filename
        default is None
        The file will be written to REFL_CODE/Files

    plt: bool, optional
        plot comes to the screen
        default is None

    outliers: bool, optional
        tried to remove outliers, but it doesn't work as yet
        default is No

    sensor: str, optional
        type of sensor, prs(for pressure), rad (for radar), flt (for float)
        default is None, which means it will print out what is there.
        if there is more than one sensor you should specifically ask for the one
        you want

    """
    # set up the address for the API call
    url1 = 'http://www.ioc-sealevelmonitoring.org/service.php?query=data&code='
    url2 = '&format=json'

    if len(date1) != 8:
        print('date1 must have 8 characters', date1); sys.exit()
    if len(date2) != 8:
        print('date2 must have 8 characters', date1); sys.exit()
    # should check for < 30 days
    if (date1[0:4] != date2[0:4]):
        print('This code does not collect data across different years.'); sys.exit()

    g.check_environ_variables()

    xdir = os.environ['REFL_CODE']
    if not os.path.exists(xdir):
        print('The REFL_CODE environment variable must be set')
        print('This will tell the code where to put the output.')
        sys.exit()

    outdir = xdir  + '/Files/'
    if not os.path.exists(outdir) :
        subprocess.call(['mkdir', outdir])

    if subdir is None:
        #print('Using this output directory: ', outdir)
        dkjflajsdf = 1
    else:
        outdir = xdir  + '/Files/' + subdir + '/'
        #print('Using this output directory: ', outdir)
        if not os.path.exists(outdir) :
            subprocess.call(['mkdir', outdir])

    csv = False
    if output is None:
        outfile = outdir + station + '_' + 'ioc.txt' # use the default
    else:
        outfile = outdir + output
        if output[-3:] == 'csv':
            csv = True

    print('WARNING: as of 2023 March 28 the output file format has changed')
    month1 = int(date1[4:6])
    month2 = int(date2[4:6])
    year = int(date1[0:4])

    if (month1 == month2):
        newurl = url1 + station + '&timestart=' + date1 + '&timestop=' + date2 + url2
        print(newurl)
        try:
            data = requests.get(newurl).json()
        except:
            print('major problem with download - internet issue?')
            sys.exit()

        NV = len(data)
        if (len(data) <= 1):
            print('No data. Exiting')
            sys.exit()
    else: 
        ij = 0
        for m in range(month1, month2+1):
            d1, d2 = find_start_stop(year, m)
            print(d1,d2)
            newurl = url1 + station + '&timestart=' + d1 + '&timestop=' + d2 + url2
            print(newurl)
            try:
                tdata = requests.get(newurl).json()
            except:
                print('major problem with download - internet issue?')
                sys.exit()

            if ij == 0:
                data = tdata
            else:
            # ? https://www.askpython.com/python/dictionary/merge-dictionaries
                data.extend(tdata)
            ij = ij + 1

        NV = len(data)
        print('number of records', NV, ij)
        if (NV <= ij):
            print('No data. Could be station does not exist.  In any case, I am exiting')
            sys.exit()

    fout = open(outfile,'w+')
    print('Writing IOC data to ', outfile)


    if csv:
        fout.write("{0:s} {1:s} \n".format('#', 'IOC Station: ' + station ))
        fout.write("# YYYY,MM,DD,HH,MM,SS,Water(m),DOY, MJD \n")
    else:
        fout.write("{0:s} \n".format('%' + ' IOC Station: ' + station ))
        fout.write("%YYYY MM DD  HH MM SS  Water(m) DOY  MJD     \n")
        fout.write("% 1   2  3   4  5  6     7      8     9  \n")
    i = 1

#    All values X where abs(X – median) > tolerance are hidden.
#    With tolerance = 3*abs(percentile90 - median)
#    The statistics (median and percentile90) are calculated on the data being plotted (12h, 1d, 7d, 30d)
    s = []
    for i in range(0, NV):
        s.append(float(data[i]['slevel']))
    medv = np.median(s) 
    print('median',medv)
    sv = np.sort(np.abs(s-medv))
    percent90 = sv[int(NV*0.9)]  

    print('Your sensor choice:', sensor)
    if outliers:
        criteria = np.abs(percent90*3)
        print('outlier criteria:',round(criteria,2), ' meters')
    else:
        criteria = 100000000

    all_sensors = []
    thetime = []; sealevel = [] ; obstimes = [] ; pt = 0
    for i in range(0, NV):
        slr = data[i]['slevel']
        instrument = data[i]['sensor']
        if instrument not in all_sensors:
            all_sensors.append(instrument)
            print('Found this sensor', instrument)
        t = data[i]['stime']
        sl = float(slr)
        if ((np.abs(sl-medv)) < criteria) and ((sensor == None) or (instrument == sensor)):
            year = int(t[0:4]); mm = int(t[5:7]); dd = int(t[8:10])
            hh = int(t[11:13]); minutes = int(t[14:16])
            sec =  int(t[17:19]) # cannot assume seconds is zero ... cause
            today = datetime.datetime(year, mm, dd)
            doy = (today - datetime.datetime(today.year, 1, 1)).days + 1
            m, f = g.mjd(year, mm, dd, hh, minutes, sec)
            mjd = m + f;
            thetime.append(mjd); sealevel.append(sl)
            #print(year,mm,dd,hh,minutes,sec)
            bigT = datetime.datetime(year=year, month=mm, day=dd, hour=hh, minute=minutes, second=sec)
            obstimes.append(bigT)

            if csv:
                fout.write(" {0:4.0f},{1:2.0f},{2:2.0f},{3:2.0f},{4:2.0f},{5:2.0f},{6:7.3f},{7:3.0f},{8:15.6f} \n".format(year, 
                    mm, dd, hh, minutes, sec, sl, doy, mjd))
            else:
                fout.write(" {0:4.0f} {1:2.0f} {2:2.0f} {3:2.0f} {4:2.0f} {5:2.0f} {6:7.3f} {7:3.0f} {8:15.6f} \n".format(year, 
                    mm, dd, hh, minutes, sec, sl, doy, mjd))

            # change format so seconds is directly after ymdhm
            #if csv:
            #    fout.write(" {0:4.0f},{1:2.0f},{2:2.0f},{3:2.0f},{4:2.0f},{5:7.3f},{6:3.0f},{7:15.6f},{8:3.0f}\n".format(year, mm, dd, hh, minutes, sl, doy, mjd,sec))
            #else:
            #    fout.write(" {0:4.0f} {1:2.0f} {2:2.0f} {3:2.0f} {4:2.0f} {5:7.3f} {6:3.0f} {7:15.6f} {8:3.0f}\n".format(year, mm, dd, hh, minutes, sl, doy, mjd,sec))
        else:
            pt = pt + 1
    fout.close()

    if plt:
        g.quickp(station,obstimes,sealevel)


def find_start_stop(year,m):
    """
    finds the start and stop times for each month of the IOC download

    Parameters
    ----------
    year : int
        full year
    m : int
        month number 

    Returns
    -------
    d1 : str
        yyyymmdd for first day of requested month
    d2 : str
        yyyymmdd for last day of requested month

    """
    cyyyy = str(year)
    cmm = '{:02d}'.format(m)
    d1 = cyyyy + cmm + '01'
    # IOC if you ask for data thru nov30, it stops at midnite nov29,
    # which is not what NOAA does - 

    if m == 12:
        d2 = str(year+1) +  '0101'
    else:
        cmm = '{:02d}'.format(m+1)
        d2 = cyyyy + cmm + '01'

    return d1, d2


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import argparse
import datetime
import wget
import os
import requests
import subprocess
import sys
import gnssrefl.gps as g

from gnssrefl.utils import validate_input_datatypes, str2bool

import gnssrefl.download_noaa as download_noaa
import gnssrefl.download_ioc as download_ioc
import gnssrefl.download_wsv as download_wsv
import gnssrefl.download_psmsl as download_psmsl

def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name, e.g. 8768094", type=str)
    parser.add_argument("network", help="tidegauge network (noaa,ioc,psmsl,wsv)", type=str)
    parser.add_argument("-date1", help="start-date, 20150101", type=str, default=None)
    parser.add_argument("-date2", help="end-date, 20150110", type=str, default=None)
    parser.add_argument("-output", default=None, help="Optional output filename", type=str)
    parser.add_argument("-plt", default=None, help="quick plot to screen", type=str)
    parser.add_argument("-datum", default=None, help="datum for NOAA", type=str)
    parser.add_argument("-sensor", default=None, help="sensor type for IOC", type=str)
    parser.add_argument("-subdir", default=None, help="optional subdirectory name for output", type=str)
    parser.add_argument("-year", default=None, help="year for archive QLD data", type=str)
    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['plt']
    args = str2bool(args, boolean_args)


    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}

def download_tides(station: str, network : str, date1: str = None, date2: str = None, output: str = None, plt: bool = False, datum: str = 'mllw', subdir: str = None, year: int=None):
    """
    Downloads tide gauge data from four different networks (see below)

    Output is written to REFL_CODE/Files/ unless subdir optional input is set. Plot is sent to the screen if requested.

    Examples
    --------

    download_tides 8768094 noaa 20210101 20210131
        NOAA station 876094

    download_tides thul ioc 20210101 20210131
        IOC station thul

    download_tides 5970026 wsv 
        WSV station 5970026

    download_tides 10313 psmsl
        PSMSL station 10313 (downloads one file)

    Parameters 
    ----------
    station : str
        station name
    network : str
        name of tide network. Options:

            noaa : US NOAA

            ioc : UNESCO

            wsv : Germany, Wasserstrassen-und Schifffahrtsverwaltung

            psmsl : Permanent Service Mean Sea Level

    date1 : str, optional
        start date, 20150101, needed for NOAA/IOC
    date2 : str,optional
        end date, 20150110, needed for NOAA/IOC
    output : str, optional
        Optional output filename
    plt : bool, optional
        plot comes to the screen
    datum: str, optional
        NOAA input, default is mllw
    sensor: str, optional
        setting for IOC
    subdir : str, optional
        subdirectory for output in the $REFL_CODE/Files area

    """
    g.check_environ_variables()

    xdir = os.environ['REFL_CODE']
    outdir = xdir  + '/Files/'
    if not os.path.exists(outdir) :
        subprocess.call(['mkdir', outdir])
    if (network == 'noaa') or (network == 'ioc'):
        if (date1 is None):
            print('You need to enter a starting date YYYYMMDD') ; sys.exit()
        if (date2 is None):
            print('You need to enter a ending date YYYYMMDD'); sys.exit()
        if len(date1) != 8:
            print('date1 must have 8 characters', date1); sys.exit()
        if len(date2) != 8:
            print('date2 must have 8 characters', date2); sys.exit()

    print(station, network, date1,date2,output,plt)
    if network == 'noaa':
        datum = 'mllw'; subdir = None
        download_noaa.download_noaa(station, date1, date2, output, plt, datum, subdir)
    elif network == 'ioc':
        outliers = False; sensor = None
        download_ioc.download_ioc(station, date1, date2, output, plt, outliers, sensor, subdir)
    elif network == 'wsv':
        download_wsv.download_wsv(station, plt, output)
    elif network == 'psmsl':
        download_psmsl.download_psmsl(station, output,plt )
    elif network == 'qld':
        download_noaa.download_qld(station,year,plt)
    else:
        print('I do not recognize your tide gauge network')


def main():
    args = parse_arguments()
    download_tides(**args)


if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
"""

"""
import argparse
import sys

import gnssrefl.gps as g

def main():
    """
    Command line tool that converts latitude, longitude, and 
    ellipsoidal ht to Cartesian coordinates  and prints to the screen

    Examples
    --------
    llh2xyz  39.949492042 -105.194266387  1728.856
        returns  -1283634.1616   -4726427.8934    4074798.0432

    Parameters
    ----------
    lat : float
        latitude in degrees
    lon : float
        longitude in degrees
    height : float
        ellipsoidal height in meters

    Returns
    -------
    XYZ : float
        Cartesian coordinates to the screen (m)
    """

    parser = argparse.ArgumentParser()
    parser.add_argument("lat", help="latitude (deg) ", type=float)
    parser.add_argument("lon", help="longitude (deg) ", type=float)
    parser.add_argument("height", help="ellipsoidal height (m) ", type=float)
    args = parser.parse_args()

    ie = False
    lat=args.lat; lon=args.lon; height=args.height
    if (lat > 90) or (lat < -90):
        print('invalid latitude. Exiting.') ; ie = True
    if (lon > 360) or (lon < -180):
        print('invalid longitude. Exiting.'); ie = True
    if (height > 10000) or (height < -1000):
        print('Very large or very small height. Exiting.'); ie = True
    if ie:
        sys.exit()

    x,y,z = g.llh2xyz(lat,lon,height)
    print("XYZ (meters) %15.4f %15.4f %15.4f " % ( x,y,z) )

if __name__ == "__main__":
    main()



import argparse
import json
import numpy as np
import os
import sys

from pathlib import Path

from gnssrefl.gps import l2c_l5_list
from gnssrefl.utils import read_files_in_dir, FileTypes, FileManagement
import gnssrefl.gnssir_v2 as guts2


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name", type=str)
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("-min_tracks", default=None, help="min number of tracks to keep mean RH (default is 100)", type=int)
    parser.add_argument("-fr", default=None, help="frequency (default is L2C)", type=int)
    parser.add_argument("-extension", default='', help="analysis extension parameter", type=str)
    parser.add_argument("-tmin", default=0.05, help="min soil texture", type=float)
    parser.add_argument("-tmax", default=0.45, help="max soil texture", type=float)
    parser.add_argument("-warning_value", default=5.5, help="warning value, phaseRMS", type=float)

    args = parser.parse_args().__dict__

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def vwc_input(station: str, year: int, fr: int = 20, min_tracks: int = 100, 
              extension : str='', tmin : float=0.05, tmax : float=0.45, minvalperday : int = 5, warning_value :float=5.5 ):
    """
    Starts the analysis for volumetric water content.  Picks up reflector height (RH) results for a 
    given station and year-year end range and computes the RH mean values and writes them 
    to a file. These will be used to compute a consistent set of phase estimates.

    Secondarily, it will add vwc input parameters into the json previously created for RH estimates.
    If nothing is requested here, it will put the defaults in the file.  

    Examples
    --------
    vwc_input p038 2018
        standard usage, station and year inputs
         
    vwc_input p038 2018 -min_tracks 50
        allow fewer values to accept a satellite track
        default is 100

    Parameters
    ----------
    station : str
        4 character ID of the station
    year : int
        full year
    fr : int, optional
        GPS frequency. Currently only supports l2c, which is frequency 20.
    min_tracks : int, optional
        number of minimum tracks needed in order to keep the average RH
    extension : str, optional
        strategy extension value (same as used in gnssir, subdaily etc)
    tmin : float, optional
        minimum soil moisture value
    tmax : float, optional
        maximum soil moisture value
    minvalperday : int, optional
        how many unique tracks are needed to compute a valid VWC measurement for that day
    warning_value : float, optional
        for removing low quality satellite tracks when running vwc

    Returns
    -------
    File with columns
    index, mean reflector_heights, satellite, average_azimuth, number of reflector heights in average, min azimuth, max azimuth

    Saves to $REFL_CODE/input/<station>_phaseRH.txt


    Saves four vwc specific parameters to existing $REFL_CODE/input/<station>.json file

    vwc_minvalperday, vwc_min_soil_texture, vwc_max_soil_texture, vwc_min_req_pts_track, vwc_warning_value

    """
    # default l2c, but can ask for L1 and L2
    xdir = Path(os.environ["REFL_CODE"]) # for kelly enloe
    myxdir = os.environ['REFL_CODE'] # for kristine 


    if (len(station) != 4):
        print('station name must be four characters. Exiting.')
        sys.exit()
    if (len(str(year)) != 4):
        print('Year must be four characters. Exiting.')
        sys.exit()

    # not sure this is needed?
    if not min_tracks:
        min_tracks = 100

    print('Minimum number of tracks required ', min_tracks)
    gnssir_results = []
    # removed the ability to look at multiple years
    # it failed and there is no need for it at this time
    y = year
    data_dir = xdir / str(y) / 'results' / station
    result_files = read_files_in_dir(data_dir)
    if result_files == None:
        print('Exiting.')
        sys.exit()

    gnssir_results = np.asarray(result_files)

    # change it to a numpy array
    #gi = np.asarray(gnssir_results)
    # I transpose this because the original code did that.
    gnssir_results = np.transpose(gnssir_results) 


    # four quadrants
    azimuth_list = [0, 90, 180, 270]

    # get the satellites for the requested frequency (20 for now) and most recent year
    if (fr == 1):
        l1_satellite_list = np.arange(1,33)
        satellite_list = l1_satellite_list
        apriori_path_f = myxdir + '/input/' + station + '_phaseRH_L1.txt'
    else:
        print('Using L2C satellite list for December 31 on ', year)
        l2c_sat, l5_sat = l2c_l5_list(year, 365)
        satellite_list = l2c_sat
        apriori_path_f = myxdir + '/input/' + station + '_phaseRH.txt'


    # window out frequency 20
    # the following function returns the index values where the statement is True
    frequency_indices = np.where(gnssir_results[10] == fr)

    reflector_height_gnssir_results = gnssir_results[2][frequency_indices]
    satellite_gnssir_results = gnssir_results[3][frequency_indices]
    azimuth_gnssir_results = gnssir_results[5][frequency_indices]

    b=0

    apriori_array = []
    for azimuth in azimuth_list:
        azimuth_min = azimuth
        azimuth_max = azimuth + 90
        for satellite in satellite_list:
            reflector_heights = reflector_height_gnssir_results[(azimuth_gnssir_results > azimuth_min)
                                                                & (azimuth_gnssir_results < azimuth_max)
                                                                & (satellite_gnssir_results == satellite)]
            azimuths = azimuth_gnssir_results[(azimuth_gnssir_results > azimuth_min)
                                              & (azimuth_gnssir_results < azimuth_max)
                                              & (satellite_gnssir_results == satellite)]
            if (len(reflector_heights) > min_tracks):
                b = b+1
                average_azimuth = np.mean(azimuths)
                #print("{0:3.0f} {1:5.2f} {2:2.0f} {3:7.2f} {4:3.0f} {5:3.0f} {6:3.0f} ".format(b, np.mean(reflector_heights), satellite, average_azimuth, len(reflector_heights),azimuth_min,azimuth_max))
                apriori_array.append([b, np.mean(reflector_heights), satellite, average_azimuth, len(reflector_heights), azimuth_min, azimuth_max])

    apriori_path = FileManagement(station, FileTypes("apriori_rh_file")).get_file_path()

    # save file

    if (len(apriori_array) == 0):
        print('Found no results - perhaps wrong year? or ')
    else:
        print('>>>> Apriori RH file written to ', apriori_path_f)
        fout = open(apriori_path_f, 'w+')
        fout.write("{0:s}  \n".format('% apriori RH values used for phase estimation'))
        l = '% year/station ' + str(year) + ' ' + station 
        fout.write("{0:s}  \n".format(l))
        fout.write("{0:s}  \n".format('% tmin 0.05 (default)'))
        fout.write("{0:s}  \n".format('% tmax 0.50 (default)'))
        fout.write("{0:s}  \n".format('% Track  RefH SatNu MeanAz  Nval   Azimuths '))
        fout.write("{0:s}  \n".format('%         m   ' ))

    #with open(apriori_path, 'w') as my_file:
        np.savetxt(fout, apriori_array, fmt="%3.0f %6.3f %4.0f %7.2f   %4.0f  %3.0f  %3.0f")
        fout.close()


    # read in existing information
    lsp = guts2.read_json_file(station, extension)

    # new one for minimum normalized amplitude
    lsp['vwc_min_norm_amp'] = 0.5;

    # save the vwc specific values that might be useful
    lsp['vwc_warning_value'] = warning_value
    lsp['vwc_min_soil_texture'] = tmin
    lsp['vwc_max_soil_texture'] = tmax
    lsp['vwc_minvalperday'] = minvalperday # this is how many unique tracks you need on each day 
    lsp['vwc_min_req_pts_track'] = min_tracks # this is total number of days needed to keep a satellite

    if len(extension)==0:
        instructions = str(os.environ['REFL_CODE']) + '/input/' + station + '.json'
    else:
        instructions = str(os.environ['REFL_CODE']) + '/input/' + station + '.' + extension + '.json'

    with open(instructions, 'w+') as outfile:
        json.dump(lsp, outfile, indent=4)


def main():
    args = parse_arguments()
    vwc_input(**args)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import numpy as np
import os
import subprocess 
import sys

def read_snr_multiday(obsfile,obsfile2,twoDays,dec=1):
    """
    originally meant to make snr arrays longer than a day to take care
    of midnight crossing.  not currently invoked.

    Snr data have units changed to linear units I believed.

    Parameters
    ----------
    obsfile : string
        name of first SNR input file

    obsfile2 : string
        name of second SNR input file

    twoDays : boolean
        False (default) for using only the first file

    dec : int
        decimation value. 1 means do nothing

    Results
    -------
    allGood1 : numpy array

    sat : numpy array
        satellite numbers 

    ele : numpy array
        elevation angle (degrees)

    azi : numpy array
        azimuth angles (degrees)

    t : numpy array
        time, seconds of the day, GPS time

    edot : numpy array
        derivative of elevation angle with respect to time

    s1 : numpy array
        SNR on L1 frequency

    s2 : numpy array
        SNR on L2 frequency

    s5 : numpy array
        SNR on L5 frequency

    s6 : numpy array
        SNR on L6 frequency

    s7 : numpy array
        SNR on L7 frequency

    s8 : numpy array
        SNR on L8 frequency

    snrE : boolean
        whether it exists

    """
#   defaults so all returned vectors have something stored in them
    sat=[]; ele =[]; azi = []; t=[]; edot=[]; s1=[];
    s2=[]; s5=[]; s6=[]; s7=[]; s8=[];
    snrE = np.array([False, True, True,False,False,True,True,True,True],dtype = bool)
#
    allGood1 = 0; allGood2 = 0
    # set these for now.  should be passed 
    # not being used ??? why are they here
    #e1 = 5
    #e2 = 15
    try:
#       this will be 24 hours - all in one calendar day 
#        print('>>>>>>>>>>>>>>>>>>>>> try to read file 1:')
        compressedObs = obsfile + '.xz'
        if (os.path.isfile(compressedObs) == True):
            print('compressed file exists, so uncompress it')
            subprocess.call(['unxz', compressedObs])
        sat, ele, azi, t, edot, s1, s2, s5, s6, s7, s8, snrE = read_one_snr(obsfile,1)
        allGood1 = 1
#        g.print_file_stats(ele,sat,s1,s2,s5,s6,s7,s8,e1,e2)
    except:
        print('>>>>> Could not read the first SNR file:', obsfile)
#
    if (dec != 1) & (allGood1 == 1):
        print('Invoking the decimation flag:')
        rem_arr = np.remainder(t, [dec])
        iss = (rem_arr==0)
        sat=sat[iss] ; ele=ele[iss] ; azi=azi[iss]
        t=t[iss] ; edot=edot[iss]
        s1= s1[iss] ; s2= s2[iss] ; 
        if len(s5) > 0:
            s5= s5[iss] ;
        if len(s6) > 0:
            s6= s6[iss] ; 
        if len(s7) > 0:
            s7= s7[iss] ; 
        if len(s8) > 0:
            s8= s8[iss]
#
    if twoDays:
#   restrict day one to first 21 hours.  will then merge iwth last three hours
#   of previous day
#       in case these observables do not exist
        Qs5=[]; Qs6=[]; Qs7=[]; Qs8=[]
        tt = t 
        hours21 = 21*3600
        print('length(tt)', len(tt))
        Qt =tt[tt < hours21]
        Qsat =sat[tt < hours21]
        Qele =ele[tt < hours21]
        Qazi =azi[tt < hours21]
        Qedot = edot[tt < hours21]
        Qs1 = s1[tt < hours21]
        Qs2 = s2[tt < hours21]
        if snrE[5]:
            Qs5 = s5[tt < hours21]
        if snrE[6]:
            Qs6 = s6[tt < hours21]
        if snrE[7]:
            Qs7 = s7[tt < hours21]
        if snrE[8]:
            Qs8 = s8[tt < hours21]
#       I think this is ok??
        QsnrE = snrE
        try:
            compressedObs = obsfile2 + '.xz'
            if (os.path.isfile(compressedObs) == True):
                print('compressed file exists, so uncompress it')
                cmd = 'unxz ' + compressedObs; os.system(cmd)
            #print('>>>>>>>>>>>>>>>>>>>>> try to read last three hours of file 2:')
            Psat, Pele, Pazi, Pt, Pedot, Ps1, Ps2, Ps5, Ps6, Ps7, Ps8, PsnrE = read_one_snr(obsfile2,2)
            allGood2 = 1
        except: 
            print('failed to read second file')
    if (twoDays) & (allGood1 == 1) & (allGood2 == 1):
        print('stack the two days')
        ele = np.hstack((Pele,Qele))
        sat = np.hstack((Psat,Qsat))
        azi = np.hstack((Pazi,Qazi))
        t =   np.hstack((Pt,Qt))
        edot= np.hstack((Pedot,Qedot))
        s1 =  np.hstack((Ps1,Qs1))
        s2 =  np.hstack((Ps1,Qs2))
        s5 =  np.hstack((Ps1,Qs5))
        s6 =  np.hstack((Ps1,Qs6))
        s7 =  np.hstack((Ps1,Qs7))
        s8 =  np.hstack((Ps1,Qs8))
    return  allGood1,sat,ele,azi,t,edot,s1,s2,s5,s6,s7,s8,snrE

def read_one_snr(obsfile,ifile):
    """
    reads a SNR file, changes units (linear) and stores as variables

    Parameters
    ----------
    obsfile : str
        SNR file name
    ifile : int
        1 for primary file or 2 for the day before the primary file

    Returns
    -------
    sat: numpy array of int 
        satellite number 
    ele : numpy array of floats 
        elevation angle in degrees
    azi : numpy array of floats
        azimuth in degrees
    t:  numpy array of floats
        time in seconds of the day
    edot : numpy array of floats
        elevation angle derivative (units?)
    s1 : numpy array of floats
        L1 SNR in dB-Hz
    s2:  numpy array of floats 
        L2 SNR in dB-Hz
    s5 :  numpy array of floats 
        L5 SNR in dB-Hz
    s6 :  numpy array of floats 
        L6 SNR in dB-Hz
    s7 :  numpy array of floats 
        L7 SNR in dB-Hz
    s8 :  numpy array of floats 
        L8 SNR in dB-Hz
    snrE : bool list
        whether the SNR exists for that Frequency 

    """

    sat=[]; ele=[]; az=[]; t=[]; edot=[]; s=[];  s2=[]; s5=[];  s6=[]; s7=[];  s8=[];
#SNR existence array : s0, s1,s2,s3,s4,s5,s6,s7,s8.  fields 0,3,4 are always false
#

    snrE = np.array([False, True, True,False,False,True,True,True,True],dtype = bool)
    f = np.genfromtxt(obsfile,comments='%')
    #print('reading from this snr file ',obsfile)
    r,c = f.shape
    if (r > 0) & (c > 0):
        #print('make sure no negative elevation angle data are used')
        i= f[:,1] > 0
        f=f[i,:]
    if r == 0:
        print('no rows in this file!')
    if c == 0:
        print('no columns in this file!')

    #print('Number of rows:', r, ' Number of columns:',c)
#   store into new variable f
#   now only keep last three hours if previous day's file
    hoursKept = 21*3600
    if (ifile == 2):
        print('window last three hours')
        tt = f[:,3]
        f=f[tt > hoursKept]
#   save satellite number, elevation angle, azimuth value
    sat = f[:,0]
    ele = f[:,1]
    azi = f[:,2]
#   negative time tags for day before
    if (ifile == 1):
        #print('keep timetags as they are ')
        t =  f[:,3]
    else:
        print('make timetags negative')
        t =  f[:,3] - 86400
#   ok - the rest is the same as alwasy
#   this is sometimes all zeros
    edot =  f[:,4]
    s1 = f[:,6]
    s2 = f[:,7]
#   typically there is a zero in this row, but older files may have something
#   something that should not be used 
    s6 = f[:,5]

    s1 = np.power(10,(s1/20))  
    s2 = np.power(10,(s2/20))  
#
    s6 = s6/20
    s6 = np.power(10,s6)  
#
#   sometimes these records exist, sometimes not
#   depends on when the file was made, which version was used
#   make sure s5 has default value?
    s5 = []
    if c > 8:
        s5 = f[:,8]
        if (sum(s5) > 0):
            s5 = s5/20; s5 = np.power(10,s5)  
    if c > 9:
        s7 = f[:,9]
        if (sum(s7) > 0):
            s7 = np.power(10,(s7/20))  
        else:
            s7 = []

    if c > 10:
        s8 = f[:,10]
        if (sum(s8) > 0):
            s8 = np.power(10,(s8/20))  
        else:
            s8 = []

    if (np.sum(s5) == 0):
        snrE[5] = False
        #print('no s5 data')
    if (np.sum(s6) == 0):
        #print('no s6 data')
        snrE[6] = False
    if (np.sum(s7) == 0):
        #print('no s7 data')
        snrE[7] = False
    if (np.sum(s8) == 0):
        snrE[8] = False
        #print('no s8 data')

    return sat, ele, azi, t, edot, s1, s2, s5, s6, s7, s8, snrE

def compress_snr_files(wantCompression, obsfile, obsfile2,TwoDays,gzip):
    """
    compresses SNR files

    Parameters
    ----------
    wantCompression : bool
        whether the file should be compressed again
    obsfile : str
        name of first SNR file
    obsfile2 : str
        name of second SNR file
    TwoDays : bool
        whether second file is being input
    gzip : bool
        whether you want to gzip/gunzip the file

    """
    if gzip:
        if (os.path.isfile(obsfile) == True):
            subprocess.call(['gzip', '-f', obsfile])
        if (os.path.isfile(obsfile2) == True and twoDays == True):
            subprocess.call(['gzip', '-f', obsfile2])
    else:
        # this is only for xz compression
        if wantCompression:
            if (os.path.isfile(obsfile) == True):
                subprocess.call(['xz', '-f', obsfile])
            if (os.path.isfile(obsfile2) == True and twoDays == True):
                subprocess.call(['xz', '-f', obsfile2])



import numpy as np
import os
import requests
import subprocess
import time

import gnssrefl.gps as g

from pathlib import Path

# https://pypi.org/project/earthscope-sdk/
from earthscope_sdk.auth.device_code_flow import DeviceCodeFlowSimple
from earthscope_sdk.auth.auth_flow import NoTokensError


def the_kelly_simple_way(url,filename):
    """
    new way to access rinex files at unavco
    using earthscope-sdk
    downloads file  - does not translate or uncompress

    Updated 2023 august 20 to place and expect the token in REFL_CODE

    Parameters
    ----------
    url : string
        path to the file
    filename : string
        rinexfilename you are downloading. Could be hatanaka or not

    Returns
    -------
    foundit : bool
        whether file was found

    """
    token_path = './'
    thedir = os.environ['REFL_CODE']
    token_path = thedir
    #print(token_path)
    device_flow = DeviceCodeFlowSimple(Path(token_path))

    #print('Seeking permission from Earthscope to use their archive - wish me luck')
    # had turned this off when nothing worked
    if True:
        try:
    # get access token from local path
            pat = 'path1'
            device_flow.get_access_token_refresh_if_necessary()
        except:
    # if no token was found locally, do the device code flow
            pat = 'path2'
            device_flow.do_flow()

    device_flow.get_access_token_refresh_if_necessary()

    s1 = time.time()
    token = device_flow.access_token
    s2 = time.time()
    #print('Time it took doing whatever ', s2-s1)

    headers = {}
    headers['authorization'] = 'Bearer ' + token
    #print('In the kelly function')

    s1 = time.time()
    r = requests.get(url, headers=headers)
    s2 = time.time()
    # Opens a local file of same name as remote file for writing to
    # check to see that the file exists
    if (r.status_code == requests.codes.ok):
        #print('File was found', filename)
        with open(filename, 'wb') as f:
            for data in r:
                f.write(data)
        #f.close() 

        foundit = True
    else:
        print('RINEX File was not found', url)
        foundit = False

    return foundit, filename



import argparse
import numpy as np
import gnssrefl.gps as g
import os
import sys


def check_l2(i,savebase,nsat_line1, lines,nlin,l2index,year,doy):
    """
    Parameters
    ----------
    i : int
        line number index in RINEX file
    savebase : str
        list of satellites at first epoch
    nsat_line1 : int
        number of satellites in first epoch
    nlin :  int
        number of lines per satellite allocated in RINEX file
    l2index : int
        index of L1 observable
    year : int
        full year
    doy : int
        day of year

    """
    l2c_sat, l5_sat = g.l2c_l5_list(year,doy)
    ii = i+2
    if (nsat_line1 > 12):
        base2 = lines[i+2][31:80].strip()
        savebase = savebase + base2
        ii = i + 3
    if (nsat_line1 > 24):
        base3 = lines[i+3][31:80].strip()
        savebase = savebase + base3
        ii = i + 4
    print('Satellites at first epoch:', savebase, '\n')
    lines2read = nsat_line1 * nlin
    isat = 0 ; l2psats = 0; foundGPS = 0 ; l2cfound = 0
    for ij in range(ii, ii+lines2read,nlin):
        if savebase[isat:isat+1] == 'G':
            foundGPS = foundGPS + 1
            GPSnum = int ( savebase[isat+1:isat+3] )
            if GPSnum in l2c_sat:
                print('found GPS satellite in L2C list ', GPSnum)
                l2cfound = l2cfound + 1

            for k in range(0,nlin):
                if (l2index < 5+k*5) and (l2index > k*5):
                    eek = l2index*16
                    l2obs = lines[ij+k][eek:eek+16]
                    if l2obs[12:15] == '   ':
                        l2psats = l2psats + 1
                        foundGPS = foundGPS - 1 # since there really was no obs
                    else: 
                        if l2obs[14:15] == ' ':
                            #print('blank')
                            okok = 1
                        else:
                            if int(l2obs[14:15]) > 0:
                                l2psats = l2psats + 1

        isat = isat + 3

    print('\nFound ', l2cfound, ' GPS satellites that could possibly be L2C in first epoch')
    print('Of those, your file has ', foundGPS-l2psats, ' valid L2C SNR observations')  

def check_rinex_file(rinexfile):
    """
    commandline tool to look at header information in a RINEX file
    tries to look for existence of L2C data

    Example
    -------

    check_rinex_file p0311520.21o

    Parameters
    ----------
    rinexfile : str
        name of the RINEX 2.11 file

    """
    last = rinexfile[-12:]
    year = 2000 + int(last[-3:-1])
    doy = int(last[4:7])
    # assume no coordinates in the file
    recx=0; recy = 0; recz = 0
    if os.path.exists(rinexfile):
        f=open(rinexfile,'r')
        lines = f.read().splitlines(True)
        lines.append('') # do not know why this is here
        eoh=0
        firstO = True
        for i,line in enumerate(lines):
            #print('reading ', line)
            eoh+=1
            base = line[0:60].strip()
            if ("END OF HEADER" in line) or (eoh > 150):

                print('Observables: ',obs)
                observables = obs.split()
                # first line of satellites
                base = lines[i+1][0:60].strip()
                savebase = lines[i+1][32:80].strip() 

                nsat_line1 = int(base[28:31])
                print('Number of satellites ' , nsat_line1)
                base2 = ''
                ii = i+2 # this is an index in the RINEX file
                if 'L2' in observables:
                    l2index = observables.index('L2')
                    print('l2index',l2index)
                    check_l2(i,savebase,nsat_line1, lines,nlin,l2index,year,doy)
                else:
                    print('\nWe cannot check for L2C unless the L2 observable is present')

                print('\n')
                if ('G' in savebase):
                    print('Found GPS Data')
                if ('E' in savebase):
                    print('Found Galileo Data')
                if ('R' in savebase): 
                    print('Found Glonass Data')
                print('\n')
                break
            else:
                desc = line[60:80].strip()
            if ('APPROX POSITION XYZ') in desc:
                print('Receiver coordinates (m):', line[:60].strip(),'\n')
                base = line[0:60]

                recx = float(base[0:14])
                recy = float(base[14:28])
                recz = float(base[28:42])
                #print(recx,recy,recz)
            if ('REC #') in desc:
                print('Receiver information: ', line[:60].strip())
            if ('ANT #') in desc:
                print('Antenna information:  ',line[:60].strip())
            if ('INTERVAL') in desc:
                print('Sampling interval    :  ',line[:60].strip())
            if ('TYPES OF OBSERV') in desc:
                thisline = line[:60].strip()
                if firstO:
                    firstO = False
                    numobs = int(line[0:6].strip())
                    print('Number of observables',numobs)
                    if (np.remainder(numobs,5) == 0 ):
                        nlin = int(numobs/5)
                    else:
                        nlin = int(numobs/5) + 1
                    #print('number of lines/sat ', nlin)
                    obs = line[6:60]
                else:
                    obs = obs + '     ' + thisline


        if ('S' not in obs):
            print('WARNING: no SNR observables in this file')
        else:
            if 'S1' in obs:
                print('L1 SNR data column found')
            if 'S2' in obs:
                print('L2 SNR data column found')
            if 'S5' in obs:
                print('L5 SNR data column found')

        if (recx == 0) or (recy == 0):
            print('WARNING: useful receiver coordinates are not provided.')
    else:
        print('File was not found ', rinexfile)
        return

def main():

    parser = argparse.ArgumentParser()
    parser.add_argument("rinexfile", help="rinexfile 2.11 name", type=str)
    args = parser.parse_args()
    rinexfile = args.rinexfile
    if os.path.exists(rinexfile):
        check_rinex_file(rinexfile)
    else:
        print('Your input file: ', rinexfile, ' does not exist ')

if __name__ == "__main__":
    main()

